//*****************************************************************************
// File: error.cpp
// Description: Nirva local error classes
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************


#include "nvcc.h"


using namespace std;


/////////////////////////////////////////////////////////////////////////////
// NvError

NvError::NvError()
{
	m_iErrorCode = NV_OK;
	m_sErrorService = "LOCAL";
	m_sErrorClass = "";
	m_sErrorInfo = "";
	m_sErrorDesc = "";
}

NvError::~NvError()
{
}

void NvError::SetError(string ErrorService, string ErrorClass, int ErrorCode, string ErrorInfo, string ErrorDesc)
{
	m_sErrorService = ErrorService;
	m_sErrorClass = ErrorClass;
	m_iErrorCode = ErrorCode;
	m_sErrorInfo = ErrorInfo;
	m_sErrorDesc = ErrorDesc;
}

void NvError::SetLocalError(string ErrorClass, int ErrorCode, string ErrorInfo, const char *Language)
{
	m_sErrorService = "LOCAL";
	m_sErrorClass = ErrorClass;
	m_iErrorCode = ErrorCode;
	m_sErrorInfo = ErrorInfo;
	m_sErrorDesc = "";
	
	string ErrorClassHere = ErrorClass;
	MakeUpper(ErrorClassHere);
	CleanString(ErrorClassHere);
	if(ErrorClassHere == "REQUEST")
	{
		switch(ErrorCode)
		{
			case NV_LOCAL_UNKNOWN_RESULT:
				m_sErrorDesc = "The command has been sent to the server but not executed";
				break;
			case NV_LOCAL_BAD_PARAMETER:
				m_sErrorDesc = "Bad function parameter";
				break;
			case NV_LOCAL_CANNOT_GET_SERVER_CONNECTION:
				m_sErrorDesc = "Cannot get connection";
				break;
			case NV_LOCAL_NETWORK_ERROR:
				m_sErrorDesc = "Network error";
				break;
			case NV_LOCAL_BAD_SERVER:
				m_sErrorDesc = "The server is not a Nirva server";
				break;
			case NV_LOCAL_SYNC_ERROR:
				m_sErrorDesc = "Synchronisation error between command and return queues";
				break;
			case NV_LOCAL_SERVER_COMMAND_ERROR:
				m_sErrorDesc = "Error in one of the commands sent to the server (more info in error for each command)";
				break;
			case NV_LOCAL_COMMAND_NOT_SENT:
				m_sErrorDesc = "This command has not been sent to the server";
				break;
			case NV_LOCAL_COMMAND_BUFFER_NOT_EMPTY:
				m_sErrorDesc = "The command buffer contains some un-sent commands";
				break;
			case NV_LOCAL_UNKNOWN_COMMAND:
				m_sErrorDesc = "Unknown command";
				break;
			case NV_LOCAL_UNKNOWN_CLASS:
				m_sErrorDesc = "Unknown class";
				break;
			case NV_LOCAL_INVALID_COMMAND_INDEX:
				m_sErrorDesc = "Invalid command index";
				break;
			case NV_LOCAL_HTTP_PROXY_AUTHENTICATION_ERROR:
				m_sErrorDesc = "Proxy http authentication error";
				break;
			case NV_LOCAL_HTTP_AUTHENTICATION_ERROR:
				m_sErrorDesc = "Host http authentication error";
				break;
			default:
				break;
		}
	}
	if(ErrorClassHere == "OBJECT")
	{
		switch(ErrorCode)
		{
			case NV_LOCAL_OBJECT_NO_TYPE:
				m_sErrorDesc = "No object type";
				break;
			case NV_LOCAL_OBJECT_UNKNOWN_TYPE:
				m_sErrorDesc = "Unknown type in the GetObject system command";
				break;
			case NV_LOCAL_OBJECT_CANNOT_CREATE:
				m_sErrorDesc = "Cannot create local object";
				break;
			case NV_LOCAL_OBJECT_NO_FILENAME:
				m_sErrorDesc = "No file name defined for a local file object";
				break;
			case NV_LOCAL_OBJECT_CANNOT_POPULATE:
				m_sErrorDesc = "Cannot populate the local object";
				break;
			case NV_LOCAL_OBJECT_CANNOT_GET:
				m_sErrorDesc = "Cannot get the local object";
				break;
			case NV_LOCAL_OBJECT_CANNOT_GET_DATA:
				m_sErrorDesc = "Cannot get the local object data";
				break;
			case NV_LOCAL_OBJECT_EVER_EXIST:
				m_sErrorDesc = "The local object ever exist";
				break;
			case NV_LOCAL_INVALID_OBJECT_INDEX:
				m_sErrorDesc = "Invalid object index";
				break;
			case NV_LOCAL_INVALID_OBJECT:
				m_sErrorDesc = "The object doesn't exist";
				break;
			case NV_LOCAL_SAME_OBJECTS:
				m_sErrorDesc = "The objects are the same";
				break;
			case NV_LOCAL_OBJECTS_NOT_COMPATIBLE:
				m_sErrorDesc = "The objects are not compatible";
				break;
			case NV_LOCAL_OBJECT_CANNOT_COPY:
				m_sErrorDesc = "Cannot copy objects";
				break;
			case NV_LOCAL_INVALID_STRINGLIST_INDEX:
				m_sErrorDesc = "Invalid string list index";
				break;
			case NV_LOCAL_INVALID_INDSTRINGLIST_KEY:
				m_sErrorDesc = "Invalid indexed string list key";
				break;
			case NV_LOCAL_TABLE_SEARCH_ERROR:
				m_sErrorDesc = "Error in search table";
				break;
			case NV_LOCAL_TABLE_CANNOT_SAVE:
				m_sErrorDesc = "Error in save table";
				break;
			case NV_LOCAL_TABLE_CANNOT_LOAD:
				m_sErrorDesc = "Error in load table";
				break;
			case NV_LOCAL_TABLE_CANNOT_IMPORT:
				m_sErrorDesc = "Error in import table";
				break;
			case NV_LOCAL_TABLE_CANNOT_EXPORT:
				m_sErrorDesc = "Error in export table";
				break;
			case NV_LOCAL_TABLE_INVALID_CELL:
				m_sErrorDesc = "Invalid cell";
				break;
			case NV_LOCAL_TABLE_CANNOT_SET_CELL_LINE:
				m_sErrorDesc = "Error in set cell line";
				break;
			case NV_LOCAL_TABLE_CANNOT_INSERT_CELL_LINE:
				m_sErrorDesc = "Error in insert cell line";
				break;
			case NV_LOCAL_TABLE_REMOVE_ROWS_ERROR:
				m_sErrorDesc = "Error in remove rows";
				break;
			case NV_LOCAL_TABLE_CLEAR_ROWS_ERROR:
				m_sErrorDesc = "Error in clear rows";
				break;
			case NV_LOCAL_TABLE_BAD_COLUMN_NAME:
				m_sErrorDesc = "Bad column name";
				break;
			case NV_LOCAL_TABLE_MODIFY_COLUMNS_ERROR:
				m_sErrorDesc = "Error in modify columns";
				break;
			case NV_LOCAL_FILE_CANNOT_CREATE:
				m_sErrorDesc = "Cannot get file size";
				break;
			case NV_LOCAL_FILE_CANNOT_CLEAR:
				m_sErrorDesc = "Cannot clear file for file object";
				break;
			case NV_LOCAL_FILE_CANNOT_APPEND:
				m_sErrorDesc = "Cannot append file to file object";
				break;
			case NV_LOCAL_FILE_NO_FILE:
				m_sErrorDesc = "No file name in file object";
				break;
			case NV_LOCAL_BINARY_CANNOT_APPEND:
				m_sErrorDesc = "Cannot append binary data";
				break;
			case NV_LOCAL_BINARY_CANNOT_SAVE:
				m_sErrorDesc = "Cannot save binary data to file";
				break;
			case NV_LOCAL_BINARY_CANNOT_LOAD:
				m_sErrorDesc = "Cannot load binary data from file";
				break;
			case NV_LOCAL_TABLE_CANNOT_SELECT_FROM:
				m_sErrorDesc = "Error in table select from";
				break;
			case NV_LOCAL_TABLE_CANNOT_SET_PRIMARY:
				m_sErrorDesc = "Cannot set the primary key";
				break;
			case NV_LOCAL_TABLE_CANNOT_SET_ROW_DATA:
				m_sErrorDesc = "Cannot set row data";
				break;
			case NV_LOCAL_OBJECT_BAD_TYPE:
				m_sErrorDesc = "Bad object type";
				break;
			case NV_LOCAL_TABLE_CANNOT_JOIN:
				m_sErrorDesc = "Cannot join tables";
				break;
			case NV_LOCAL_TABLE_CANNOT_ADD:
				m_sErrorDesc = "Cannot add tables";
				break;
			case NV_LOCAL_TABLE_CANNOT_GET_ROW_INDEX:
				m_sErrorDesc = "Cannot get row index from primary value";
				break;
			case NV_LOCAL_TABLE_CANNOT_ADD_RECORDS:
				m_sErrorDesc = "Cannot add records to the table";
				break;
			default:
				break;
		}
	}
}
