//*****************************************************************************
// File: util.h
// Description: Useful functions
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifndef _UTIL_H
#define _UTIL_H

#ifdef WIN32
#include <windows.h>
#endif

#include <string>
#include <vector>


#ifndef WIN32
#ifndef HMODULE
#define HMODULE void *
typedef int (*FARPROC)();
#endif
#endif


int CopyFileEx(const char *SourceFile, const char *DestFile, int Offset=0, int BytesToCopy = -1);
int CopyFileEx(FILE *FileSource, FILE *FileDest, int Offset=0, int BytesToCopy = -1);
int AppendFileEx(const char *SourceFile, const char *DestFile, int Offset=0, int BytesToCopy = -1);
int AppendFileEx(FILE *FileSource, FILE *FileDest, int Offset=0, int BytesToCopy = -1);
bool GetTemporaryFile(std::string &FileName, std::string Dir, std::string Extension, std::string Prefix, std::string Suffix);
void RemoveFile(const char *FileName);
int FileExist(const char *FileName);
int DirExist(const char *DirName);
bool DirList(const char *DirName, const char *Root, std::vector<std::string> *List);
bool IsDirEmpty(const char *DirName);
bool FileList(const char *DirName, const char *Root, std::vector<std::string> *List);
bool CreateDir(const char *DirName);
bool RemoveDir(const char *DirName, bool Self = true, const char *FileRoot = "*", const char *SubDirRoot = "*");
bool GetNoCaseRealFileName(std::string& FileName);
int freadline(char *Line, int LineLength, FILE *stream);
char *strstrnocase(const char *string, const char *strCharSet);
int strcmpnocase(const char *string1, const char *string2);
void CleanString(char *String);
void CleanString(std::string& StringToClean);
void AddDirSuffix(std::string& DirName);
void AddEndLine(std::string& String);
void RemoveDirSuffix(std::string& DirName);
bool CompareWorkDir(std::string Dir1, std::string Dir2);
void RemoveFirstAndEndBlanks(std::string& String);
void RemoveDoubleQuotes(std::string& String);
void SetDirName(std::string& DirName);
void GetFromProfile(const char *FileName, char *Output, unsigned int size, const char *Section, const char *Entry, const char *Default);
void RemoveFiles(const char *WorkDir, const char *Root);
std::string GetFirstFileDir(const char *DirName, const char *Root);
unsigned int GetAllocatedMemory();
int ExecPrgm(const char *CommandLine, int *RetCode, int TimeOut);
bool ArrangeDate(std::string& DateToArrange);
bool ArrangeHour(std::string& HourToArrange);
#ifndef WIN32
char *strlwr(char *string);	// These very usefull functions unfornatly doesn't exist on unix
char *strupr(char *string);	// So, we do them
#endif
HMODULE MyLoadLibrary(const char *LibraryPath);
void MyFreeLibrary(HMODULE hModule);
FARPROC MyGetLibFunction(HMODULE hModule, const char *FunctionName);
std::string EncodeStringForXML(std::string StringToEncode);
void SetAsciiFile(const char *FileName);

#endif //_UTIL_H
