// edotnet.cpp : implementation file
//


#include "stdafx.h"
#include <windows.h>
#include <vcclr.h>
#include "edotnet.h"
#include "nvsext.h"
#include "../nvs/nvdotnet.h"
#include <string>

#using "mscorlib.dll"
using namespace System;
using namespace System::Reflection;
using namespace System::Security::Policy;
using namespace System::Collections;
using namespace System::Threading;
using namespace System::Text;
using namespace System::Runtime::InteropServices;
using namespace Nirva;

using namespace std;

#define RESULT_BUFFER_LENGTH	1024

string sNirvaDir;

// NIRVA entry point functions
extern "C"
{
	int NvsCommand(void *pCommand);
	int NvsRender(void *RenderInterface, const char *ServiceName);
}

int NvInitService(NvServiceCommand *Command, string& ServiceName);			// Service init
int NvExitService(NvServiceCommand *Command, string& ServiceName);			// Service exit
int NvCommand(NvServiceCommand *Command, string& ServiceName);		// Real command
int NvRender(NvRendererInt *Render, string& ServiceName);					// Service renderer


#ifdef WIN32
#pragma unmanaged
// Main dll entry point for windows
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
		
	return 1;   // ok
}
#endif
#pragma managed






public ref class NvDotnetEnv
{
protected:

	!NvDotnetEnv();

public:
	ServiceEntryPoint^ obj;
	String^ ServiceName;

	NvDotnetEnv();
	~NvDotnetEnv();

	void Clear();
	int LoadClass(String^ DirName, String^ ClassName);
	void UnLoadClass();
};

public ref class NvDotnetEnvList
{
private:
	Hashtable^ Environments;
	Object^ ProtectEnvironments;		// USe for thread locking of the table

public:
	NvDotnetEnvList();

	// Get an item and it if not existing
	void AddDotnetEnv(String^ ServiceName, NvDotnetEnv^ DotnetEnv);

	// Remove the NvDotnetEnv corresponding to the service
	void RemoveDotnetEnv(String^ ServiceName);

	// Get the NvDotnetEnv corresponding to the service
	NvDotnetEnv^ GetDotnetEnv(String^ ServiceName);

	// Remove all items from the list
	void RemoveAll();

	// Clear all environments and reset the hash table
	void Clear();
};



public ref class NvAppDomains
{
private:
	Hashtable^ Domains;
	Object^ ProtectDomains;		// USe for thread locking of the table

public:
	NvAppDomains();

	// Get an item and it if not existing
	AppDomain^ GetAdd(String^ Type, String^Name);

	// remove an item
	void Remove(String^ Type, String^Name);

	// Clear all domains and reset the hash table
	void Clear();
};


// Implementation of the Nirva interface to service
public ref class NvCmd : public MarshalByRefObject, public Nirva::nvcmd
{
private:
	NvServiceCommand* NirvaCommand;
	String^ LastResult;
	char *ResultBuffer;							// command result buffer
	int ResultBufferSize;						// command result buffer size

	int SetBufferSize(int DataSize);

protected:
	!NvCmd();

public:
	String^ sServiceName;

	NvCmd();
	
	NvCmd(void *pCommand, String^ ServiceName);
	~NvCmd();

	void Clear();

	virtual void SetError(String^ ErrorClass, int ErrorCode, String^ ErrorInfo);
	virtual bool Command(String^ CommandString);
	virtual String^ GetResult();
	virtual String^ GetSessionId();
	virtual String^ GetClass();
	virtual String^ GetCommand();
	virtual String^ GetInContainer();
	virtual String^ GetOutContainer();
	virtual String^ GetMachineName();
	virtual String^ GetMachineUser();
	virtual String^ GetLanguage();
	virtual String^ GetError(String^ InfoType);
	virtual int GetNumParameters();
	virtual String^ GetSource();
	virtual bool IsCommand(String^ Class, String^ Command, String^ Source);
	virtual bool ParameterExist(String^ ParameterName);
	virtual String^ GetParameter(String^ ParameterName);
	virtual void AddHttpHeader(String^ headerName, String^ headerValue);
	virtual String^ GetParameterName(int Index);
};


// Implementation of the Nirva interface to procedure
public ref class NvsInt : public MarshalByRefObject, public Nirva::nvsint
{
private:
	NvDotnetCommand* NirvaCommand;
	String^ LastResult;
	char *ResultBuffer;							// command result buffer
	int ResultBufferSize;						// command result buffer size

	int SetBufferSize(int DataSize);

protected:
	!NvsInt();

public:

	NvsInt();
	NvsInt(void *pCommand);
	~NvsInt();

	void Clear();

	virtual void SetError(String^ ErrorInfo);
	virtual void SetErrorEx(String^ ErrorService, String^ ErrorClass, int ErrorCode, String^ ErrorInfo);
	virtual int Command(String^ CommandString);
	virtual String^ GetResult();
	virtual String^ GetSessionId();
	virtual String^ GetService();
	virtual String^ GetClass();
	virtual String^ GetCommand();
	virtual String^ GetInContainer();
	virtual String^ GetOutContainer();
	virtual String^ GetLanguage();
	virtual String^ GetError(String^ InfoType);
	virtual int GetNumParameters();
	virtual String^ GetSource();
	virtual bool ParameterExist(String^ ParameterName);
	virtual String^ GetParameter(String^ ParameterName);
	virtual String^ GetParameterName(int Index);
};

public ref class StringConvert
{
public:

	static String^ NewStringUTF(LPCSTR MyString);
	static LPSTR GetStringUTFChars(String^ MyString);
	static void ReleaseStringUTFChars(LPSTR MyString);
};


// A global class maintains the AppDomain list
// There is one AppDomain for each application, service, web service, and system
// The AppDomain is created on request and unloaded when vie the NvDotnetUnloadDomain function
// Using specific app domains allow to install .net files (dlls) when the applicaiton, service or web service is stopped
public ref class Globals abstract sealed {
public:
   static NvAppDomains Domains;
   static NvDotnetEnvList Environments;
};


String^ StringConvert::NewStringUTF(LPCSTR MyString)
{
	if(MyString == NULL)
		return (gcnew String(""));
	int StringSize = (int)(strlen(MyString));
	if(!StringSize)
		return (gcnew String(""));
	int Size = MultiByteToWideChar(CP_UTF8, 0, MyString, -1, NULL, 0);
	LPWSTR WideChars = new WCHAR[Size];
	MultiByteToWideChar(CP_UTF8, 0, MyString, -1, WideChars, Size);
	String^ Result = gcnew String(WideChars);
	delete WideChars;
	return Result;
}

LPSTR StringConvert::GetStringUTFChars(String^ MyString)
{
	int ResultBufferSize = 0;
	pin_ptr<const wchar_t> WideChars = PtrToStringChars(MyString);
	int Size = WideCharToMultiByte(CP_UTF8, 0, WideChars, -1, NULL, 0, NULL, NULL);
	LPSTR Chars = (LPSTR)malloc(Size);
	if(Chars != NULL)
		WideCharToMultiByte(CP_UTF8, 0, WideChars, -1, Chars, Size, NULL, NULL);
	return Chars;
}

void StringConvert::ReleaseStringUTFChars(LPSTR MyString)
{
	if(MyString != NULL)
		free(MyString);
}



// Implementation of the Nirva interface to procedure
int NvsInt::SetBufferSize(int DataSize)
{
	if(DataSize < ResultBufferSize)
		return 1;

	if(ResultBufferSize == 0)
	{
		ResultBuffer = (char *)malloc(RESULT_BUFFER_LENGTH);
		if(ResultBuffer == NULL)
			return 0;
		ResultBufferSize = RESULT_BUFFER_LENGTH;
	}

	if(DataSize < ResultBufferSize)
		return 1;

	char *NewResultBuffer = (char *)realloc(ResultBuffer, DataSize+1);
	if(NewResultBuffer == NULL)
		return 0;

	ResultBuffer = NewResultBuffer;
	ResultBufferSize = DataSize+1;

	return 2;
}

NvsInt::!NvsInt()
{
	Clear();
}


NvsInt::NvsInt()
{
	NirvaCommand = NULL;
	LastResult = "";
	ResultBufferSize = 0;
	SetBufferSize(0);
}

NvsInt::NvsInt(void *pCommand)
{
	NirvaCommand = (NvDotnetCommand*)pCommand;
	LastResult = "";
	ResultBufferSize = 0;
	SetBufferSize(0);
}
NvsInt::~NvsInt()
{
	this->!NvsInt();
}

void NvsInt::Clear()
{
	if(ResultBufferSize)
		free(ResultBuffer);
	ResultBufferSize = 0;
}

void NvsInt::SetError(String^ ErrorInfo)
{
	try
	{
		if(NirvaCommand == NULL)
			return;
		LPSTR str = StringConvert::GetStringUTFChars(ErrorInfo);
		string ErrorInfoHere = str;
		StringConvert::ReleaseStringUTFChars(str);

		NirvaCommand->SetError(ErrorInfoHere.c_str());
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (SetError) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (SetError)|");
	}
}

void NvsInt::SetErrorEx(String^ ErrorService, String^ ErrorClass, int ErrorCode, String^ ErrorInfo)
{
	try
	{
		if(NirvaCommand == NULL)
			return;
		LPSTR str = StringConvert::GetStringUTFChars(ErrorService);
		string ErrorServiceHere = str;
		StringConvert::ReleaseStringUTFChars(str);
		
		str = StringConvert::GetStringUTFChars(ErrorClass);
		string ErrorClassHere = str;
		StringConvert::ReleaseStringUTFChars(str);
		
		str = StringConvert::GetStringUTFChars(ErrorInfo);
		string ErrorInfoHere = str;
		StringConvert::ReleaseStringUTFChars(str);

		NirvaCommand->SetErrorEx(ErrorServiceHere.c_str(), ErrorClassHere.c_str(), ErrorCode, ErrorInfoHere.c_str());
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (SetErrorEx) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (SetErrorEx)|");
	}
}
int NvsInt::Command(String^ CommandString)
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return 0;
		LPSTR str = StringConvert::GetStringUTFChars(CommandString);
		string CommandStringHere = str;
		StringConvert::ReleaseStringUTFChars(str);
	
		ResultBuffer[0] = '\0';
		int DataSize;
		if(!NirvaCommand->Command(CommandStringHere.c_str(), ResultBuffer, ResultBufferSize, &DataSize))
			return 0;

		int ResultSetBufferSize = SetBufferSize(DataSize);
		if(ResultSetBufferSize == 0)
			return 0;	// Error
		if(ResultSetBufferSize == 2)
		{
			// We got a reallocation so we must get the last output buffer
			NirvaCommand->Command("NV_CMD=|COMMAND:GET_LAST_OUTPUT_BUFFER|", ResultBuffer, ResultBufferSize, &DataSize);
//			if(!NirvaCommand->Command(CommandStringHere.c_str(), ResultBuffer, ResultBufferSize, &DataSize))
//				return 0;
		}

		LastResult = StringConvert::NewStringUTF(ResultBuffer);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (Command) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (Command)|");
		Error = true;
	}
	if(Error)
		return 0;
	return 1;
}

String^ NvsInt::GetResult()
{
	return LastResult;
}

String^ NvsInt::GetSessionId()
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
		int DataSize;
		NirvaCommand->GetSessionId(ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetSessionId(ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetSessionId) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetSessionId)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}

String^ NvsInt::GetService()
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
		int DataSize;
		NirvaCommand->GetService(ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetService(ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetService) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetService)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}

String^ NvsInt::GetClass()
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
		int DataSize;
		NirvaCommand->GetClass(ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetClass(ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetClass) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetClass)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}

String^ NvsInt::GetCommand()
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
		int DataSize;
		NirvaCommand->GetCommand(ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetCommand(ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetCommand) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetCommand)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}

String^ NvsInt::GetInContainer()
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
		int DataSize;
		NirvaCommand->GetInContainer(ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetInContainer(ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetInContainer) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetInContainer)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}

String^ NvsInt::GetOutContainer()
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
		int DataSize;
		NirvaCommand->GetOutContainer(ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetOutContainer(ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetOutContainer) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetOutContainer)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}

String^ NvsInt::GetLanguage()
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
		int DataSize;
		NirvaCommand->GetLanguage(ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetLanguage(ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetLanguage) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetLanguage)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}

String^ NvsInt::GetError(String^ InfoType)
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
	
		LPSTR str = StringConvert::GetStringUTFChars(InfoType);
		string InfoTypeHere = str;
		StringConvert::ReleaseStringUTFChars(str);
		
		int DataSize;
		NirvaCommand->GetError(InfoTypeHere.c_str(), ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetError(InfoTypeHere.c_str(), ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetError) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetError)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}

int NvsInt::GetNumParameters()
{
	int Result = 0;
	try
	{
		if(NirvaCommand == NULL)
			return 0;
		Result = NirvaCommand->GetNumParameters();
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetNumParameters) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetNumParameters)|");
	}
	
	return Result;
}

String^ NvsInt::GetSource()
{
	string Result = "UNKNOWN";
	try
	{
		if(NirvaCommand == NULL)
			return StringConvert::NewStringUTF(Result.c_str());
		int SourceType = NirvaCommand->GetSource();
		switch(SourceType)
		{
			case NV_SOURCE_CLIENT:
				Result = "CLIENT";
				break;
			case NV_SOURCE_BROWSER:
				Result = "BROWSER";
				break;
			case NV_SOURCE_PROCEDURE:
				Result = "PROCEDURE";
				break;
			case NV_SOURCE_SERVICE:
				Result = "SERVICE";
				break;
			case NV_SOURCE_SERVER:
				Result = "SERVER";
				break;
		}
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetSource) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetSource)|");
	}

	return StringConvert::NewStringUTF(Result.c_str());
}

bool NvsInt::ParameterExist(String^ ParameterName)
{
	bool Result = false;
	try
	{
		if(NirvaCommand == NULL)
			return false;
	
		LPSTR str = StringConvert::GetStringUTFChars(ParameterName);
		string ParameterNameHere = str;
		StringConvert::ReleaseStringUTFChars(str);
		
		Result = NirvaCommand->ParameterExist(ParameterNameHere.c_str());
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (ParameterExist) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (ParameterExist)|");
	}
	
	return Result;
}

String^ NvsInt::GetParameter(String^ ParameterName)
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
	
		LPSTR str = StringConvert::GetStringUTFChars(ParameterName);
		string ParameterNameHere = str;
		StringConvert::ReleaseStringUTFChars(str);
		
		int DataSize;
		NirvaCommand->GetParameter(ParameterNameHere.c_str(), ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetParameter(ParameterNameHere.c_str(), ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetParameter) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetParameter)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}

String^ NvsInt::GetParameterName(int Index)
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
		
		int DataSize;
		NirvaCommand->GetParameterName(Index, ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetParameterName(Index, ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetParameterName) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetParameterName)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}






int NvCmd::SetBufferSize(int DataSize)
{
	if(DataSize < ResultBufferSize)
		return 1;

	if(ResultBufferSize == 0)
	{
		ResultBuffer = (char *)malloc(RESULT_BUFFER_LENGTH);
		if(ResultBuffer == NULL)
			return 0;
		ResultBufferSize = RESULT_BUFFER_LENGTH;
	}

	if(DataSize < ResultBufferSize)
		return 1;

	char *NewResultBuffer = (char *)realloc(ResultBuffer, DataSize+1);
	if(NewResultBuffer == NULL)
		return 0;

	ResultBuffer = NewResultBuffer;
	ResultBufferSize = DataSize+1;

	return 2;
}


NvCmd::!NvCmd()
{
	Clear();
}

NvCmd::NvCmd()
{
	NirvaCommand = NULL;
	LastResult = "";
	ResultBufferSize = 0;
	SetBufferSize(0);
	sServiceName = "";
}

NvCmd::NvCmd(void *pCommand, String^ ServiceName)
{
	NirvaCommand = (NvServiceCommand*)pCommand;
	LastResult = "";
	ResultBufferSize = 0;
	SetBufferSize(0);
	sServiceName = ServiceName;
}
NvCmd::~NvCmd()
{
	this->!NvCmd();
}

void NvCmd::Clear()
{
	if(ResultBufferSize)
		free(ResultBuffer);
	ResultBufferSize = 0;
}

void NvCmd::SetError(String^ ErrorClass, int ErrorCode, String^ ErrorInfo)
{
	try
	{
		if(NirvaCommand == NULL)
			return;
		
		LPSTR str = StringConvert::GetStringUTFChars(ErrorClass);
		string ErrorClassHere = str;
		StringConvert::ReleaseStringUTFChars(str);
		
		str = StringConvert::GetStringUTFChars(ErrorInfo);
		string ErrorInfoHere = str;
		StringConvert::ReleaseStringUTFChars(str);

		NirvaCommand->SetError(ErrorClassHere.c_str(), ErrorCode, ErrorInfoHere.c_str());
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (SetError) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (SetError)|");
	}
}

bool NvCmd::Command(String^ CommandString)
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return false;
		LPSTR str = StringConvert::GetStringUTFChars(CommandString);
		string CommandStringHere = str;
		StringConvert::ReleaseStringUTFChars(str);
	
		ResultBuffer[0] = '\0';
		int DataSize;
		if(!NirvaCommand->Command(CommandStringHere.c_str(), ResultBuffer, ResultBufferSize, &DataSize))
			return false;

		int ResultSetBufferSize = SetBufferSize(DataSize);
		if(ResultSetBufferSize == 0)
			return false;	// Error
		if(ResultSetBufferSize == 2)
		{
			// We got a reallocation so we must get the last output buffer
			NirvaCommand->Command("NV_CMD=|COMMAND:GET_LAST_OUTPUT_BUFFER|", ResultBuffer, ResultBufferSize, &DataSize);
//			if(!NirvaCommand->Command(CommandStringHere.c_str(), ResultBuffer, ResultBufferSize, &DataSize))
//				return false;
		}

		LastResult = StringConvert::NewStringUTF(ResultBuffer);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (Command) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (Command)|");
		Error = true;
	}
	if(Error)
		return false;
	return true;
}

String^ NvCmd::GetResult()
{
	return LastResult;
}

String^ NvCmd::GetSessionId()
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
		int DataSize;
		NirvaCommand->GetSessionId(ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetSessionId(ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetSessionId) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetSessionId)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}

String^ NvCmd::GetClass()
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
		int DataSize;
		NirvaCommand->GetClass(ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetClass(ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetClass) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetClass)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}

String^ NvCmd::GetCommand()
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
		int DataSize;
		NirvaCommand->GetCommand(ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetCommand(ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetCommand) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetCommand)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}

String^ NvCmd::GetInContainer()
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
		int DataSize;
		NirvaCommand->GetInContainer(ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetInContainer(ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetInContainer) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetInContainer)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}

String^ NvCmd::GetOutContainer()
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
		int DataSize;
		NirvaCommand->GetOutContainer(ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetOutContainer(ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetOutContainer) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetOutContainer)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}

String^ NvCmd::GetMachineName()
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
		int DataSize;
		NirvaCommand->GetMachineName(ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetMachineName(ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetMachineName) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetMachineName)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}

String^ NvCmd::GetMachineUser()
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
		int DataSize;
		NirvaCommand->GetMachineUser(ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetMachineUser(ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetMachineUser) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetMachineUser)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}

String^ NvCmd::GetLanguage()
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
		int DataSize;
		NirvaCommand->GetLanguage(ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetLanguage(ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetLanguage) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetLanguage)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}

String^ NvCmd::GetError(String^ InfoType)
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
	
		LPSTR str = StringConvert::GetStringUTFChars(InfoType);
		string InfoTypeHere = str;
		StringConvert::ReleaseStringUTFChars(str);
		
		int DataSize;
		NirvaCommand->GetError(InfoTypeHere.c_str(), ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetError(InfoTypeHere.c_str(), ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetError) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetError)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}

int NvCmd::GetNumParameters()
{
	int Result = 0;
	try
	{
		if(NirvaCommand == NULL)
			return 0;
		Result = NirvaCommand->GetNumParameters();
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetNumParameters) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetNumParameters)|");
	}
	
	return Result;
}

String^ NvCmd::GetSource()
{
	string Result = "UNKNOWN";
	try
	{
		if(NirvaCommand == NULL)
			return StringConvert::NewStringUTF(Result.c_str());
		int SourceType = NirvaCommand->GetSource();
		switch(SourceType)
		{
			case NV_SOURCE_CLIENT:
				Result = "CLIENT";
				break;
			case NV_SOURCE_BROWSER:
				Result = "BROWSER";
				break;
			case NV_SOURCE_PROCEDURE:
				Result = "PROCEDURE";
				break;
			case NV_SOURCE_SERVICE:
				Result = "SERVICE";
				break;
			case NV_SOURCE_SERVER:
				Result = "SERVER";
				break;
		}
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetSource) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetSource)|");
	}

	return StringConvert::NewStringUTF(Result.c_str());
}

bool NvCmd::IsCommand(String^ Class, String^ Command, String^ Source)
{
	bool Result = false;
	try
	{
		if(NirvaCommand == NULL)
			return false;
	
		LPSTR str = StringConvert::GetStringUTFChars(Class);
		string ClassHere = str;
		StringConvert::ReleaseStringUTFChars(str);
		
		str = StringConvert::GetStringUTFChars(Command);
		string CommandHere = str;
		StringConvert::ReleaseStringUTFChars(str);
		
		str = StringConvert::GetStringUTFChars(Source);
		string SourceHere = str;
		StringConvert::ReleaseStringUTFChars(str);
		
		const char *pClassHere = NULL;
		const char *pCommandHere = NULL;
		int iSource = -1;
		if(SourceHere == "CLIENT")
			iSource = NV_SOURCE_CLIENT;
		if(SourceHere == "BROWSER")
			iSource = NV_SOURCE_BROWSER;
		if(SourceHere == "PROCEDURE")
			iSource = NV_SOURCE_PROCEDURE;
		if(SourceHere == "SERVICE")
			iSource = NV_SOURCE_SERVICE;
		if(SourceHere == "SERVER")
			iSource = NV_SOURCE_SERVER;

		if(ClassHere.length())
			pClassHere = ClassHere.c_str();

		if(CommandHere.length())
			pCommandHere = CommandHere.c_str();
		
		Result = NirvaCommand->IsCommand(pClassHere, pCommandHere, iSource);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (IsCommand) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (IsCommand)|");
		Result = false;
	}
	
	return Result;
}

bool NvCmd::ParameterExist(String^ ParameterName)
{
	bool Result = false;
	try
	{
		if(NirvaCommand == NULL)
			return false;
	
		LPSTR str = StringConvert::GetStringUTFChars(ParameterName);
		string ParameterNameHere = str;
		StringConvert::ReleaseStringUTFChars(str);
		
		Result = NirvaCommand->ParameterExist(ParameterNameHere.c_str());
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (ParameterExist) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (ParameterExist)|");
		Result = false;
	}
	
	return Result;
}

String^ NvCmd::GetParameter(String^ ParameterName)
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
	
		LPSTR str = StringConvert::GetStringUTFChars(ParameterName);
		string ParameterNameHere = str;
		StringConvert::ReleaseStringUTFChars(str);
		
		int DataSize;
		NirvaCommand->GetParameter(ParameterNameHere.c_str(), ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetParameter(ParameterNameHere.c_str(), ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetParameter) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetParameter)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}

void NvCmd::AddHttpHeader(String^ headerName, String^ headerValue)
{
	try
	{
		if(NirvaCommand == NULL)
			return;
	
		LPSTR str = StringConvert::GetStringUTFChars(headerName);
		string headerNameHere = str;
		StringConvert::ReleaseStringUTFChars(str);
		
		str = StringConvert::GetStringUTFChars(headerValue);
		string headerValueHere = str;
		StringConvert::ReleaseStringUTFChars(str);
		
		NirvaCommand->AddHttpHeader(headerNameHere.c_str(), headerValueHere.c_str());
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (IsCommand) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (IsCommand)|");
	}
}

String^ NvCmd::GetParameterName(int Index)
{
	bool Error = false;
	try
	{
		if(NirvaCommand == NULL)
			return (gcnew String(""));
		
		int DataSize;
		NirvaCommand->GetParameterName(Index, ResultBuffer, ResultBufferSize, &DataSize);
		if(SetBufferSize(DataSize) == 2)
			NirvaCommand->GetParameterName(Index, ResultBuffer, ResultBufferSize, &DataSize);
	}
	catch(...)
	{
		printf("*** Exception occured in dotnet interface (GetParameterName) ***\n");
		NirvaCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in dotnet interface (GetParameterName)|");
		Error = true;
	}
	
	if(Error)
		return (gcnew String(""));
	return StringConvert::NewStringUTF(ResultBuffer);
}



NvAppDomains::NvAppDomains()
{
	ProtectDomains = gcnew Object;
	Domains = gcnew Hashtable();
}

// Get an item and it if not existing
AppDomain^ NvAppDomains::GetAdd(String^ Type, String^Name)
{
	String^ Key = Type + "::" + Name;

	Monitor::Enter(ProtectDomains);
	AppDomain^ domain = nullptr;
	if(Domains->Contains(Key))
	{	
		domain = (AppDomain^)Domains[Key];
		Monitor::Exit(ProtectDomains);
		return domain;
	}
	
	String^ DomainName = Key;	// Domain name set to the key
	String^ BaseDir = gcnew String(sNirvaDir.c_str());	// Base directory is the Nirva root because assemblies must be in subdirectories
	String^ BinPath = "Bin;";	// Nirva/Bin is added because contains the nirvadn.dll that defines interfaces 
	if(Type == DOMAIN_TYPE_APPLICATION)
		BinPath += "Applications\\" + Name + "\\Procs"; 
	if(Type == DOMAIN_TYPE_SERVICE)
		BinPath += "Services\\" + Name + "\\Procs"; 
	if(Type == DOMAIN_TYPE_WEBSERVICE)
		BinPath += "Webervices\\" + Name + "\\Procs"; 
	if(Type == DOMAIN_TYPE_SYSTEM)
		BinPath += "Procs";
	
	AppDomainSetup ^setup = gcnew AppDomainSetup();
	setup->ApplicationBase = BaseDir;
	setup->ApplicationName = DomainName;
	setup->PrivateBinPath = BinPath;


	// Create domain
	try
	{
		domain = AppDomain::CreateDomain(DomainName, nullptr, setup);
	}
	catch(Exception^ e)
	{
		Console::Write("Error CreateDomain: ");
		Console::WriteLine(e->Message);
		Monitor::Exit(ProtectDomains);
		return nullptr;
	}
	if(domain == nullptr)
	{
		Console::WriteLine("Error CreateDomain");
		Monitor::Exit(ProtectDomains);
		return nullptr;
	}
	Domains->Add(Key, domain);
	Monitor::Exit(ProtectDomains);
	return domain;
}

// remove an item
void NvAppDomains::Remove(String^ Type, String^Name)
{
	String^ Key = Type + "::" + Name;
	Monitor::Enter(ProtectDomains);
	if(!Domains->Contains(Key))
	{	
		Monitor::Exit(ProtectDomains);
		return;
	}
	AppDomain^ domain = (AppDomain^)Domains[Key];
	Domains->Remove(Key);
	if(domain != nullptr)
	{
		try
		{
			AppDomain::Unload(domain);
		}
		catch(Exception^ e)
		{
			Console::Write("Error UnlodDomain: ");
			Console::WriteLine(e->Message);
		}
	}
	Monitor::Exit(ProtectDomains);
}

// Clear all domains and reset the hash table
void NvAppDomains::Clear()
{
	Monitor::Enter(ProtectDomains);
	IDictionaryEnumerator^ Enumerator = Domains->GetEnumerator();
	while (Enumerator->MoveNext())
	{
		AppDomain^ domain = (AppDomain^)(Enumerator->Value);
		if(domain != nullptr)
		{
			try
			{
				AppDomain::Unload(domain);
			}
			catch(Exception^ e)
			{
				Console::Write("Error UnlodDomain: ");
				Console::WriteLine(e->Message);
			}
		}
	}
	Domains->Clear();
	Monitor::Exit(ProtectDomains);
}





NvDotnetEnv::!NvDotnetEnv()
{
	Clear();
}


NvDotnetEnv::NvDotnetEnv()
{
	ServiceName = "";
	obj = nullptr;
}
NvDotnetEnv::~NvDotnetEnv()
{
	this->!NvDotnetEnv();
}

void NvDotnetEnv::Clear()
{
	UnLoadClass();
}


int NvDotnetEnv::LoadClass(String^ DirName, String^ ClassName)
{
	// First unload the class if ever loaded
	UnLoadClass();

	//PM Todo


	String^ Type = gcnew String("SERVICE");

	// Domain init
	AppDomain^ domain = Globals::Domains.GetAdd(Type, ServiceName);
	if(domain == nullptr)
		return 0;

	// get an instance of the class
	try
	{
		obj = (ServiceEntryPoint^) domain->CreateInstanceFromAndUnwrap(DirName, ClassName);
	}
	catch(Exception^ e)
	{
		Console::Write("Error CreateInstanceFromAndUnwrap: ");
		Console::WriteLine(e->Message);
		return 0;
	}

	return 1;
}


void NvDotnetEnv::UnLoadClass()
{
	if(obj != nullptr)
	{
		delete obj;
		obj = nullptr;
	}
}




NvDotnetEnvList::NvDotnetEnvList()
{
	ProtectEnvironments = gcnew Object;
	Environments = gcnew Hashtable();
}

// Get an item and it if not existing
void NvDotnetEnvList::AddDotnetEnv(String^ ServiceName, NvDotnetEnv^ DotnetEnv)
{
	Monitor::Enter(ProtectEnvironments);
	NvDotnetEnv^ env = nullptr;
	if(Environments->Contains(ServiceName))
	{
		Environments->Remove(ServiceName);
		env = (NvDotnetEnv^)Environments[ServiceName];
		delete env;
	}
	Environments->Add(ServiceName, DotnetEnv);
	Monitor::Exit(ProtectEnvironments);
}


// Remove the NvDotnetEnv corresponding to the service
void NvDotnetEnvList::RemoveDotnetEnv(String^ ServiceName)
{
	Monitor::Enter(ProtectEnvironments);
	if(!Environments->Contains(ServiceName))
	{	
		Monitor::Exit(ProtectEnvironments);
		return;
	}
	NvDotnetEnv^ env = (NvDotnetEnv^)Environments[ServiceName];
	Environments->Remove(ServiceName);
	delete env;
	Monitor::Exit(ProtectEnvironments);
}

// Get the NvDotnetEnv corresponding to the service
NvDotnetEnv^ NvDotnetEnvList::GetDotnetEnv(String^ ServiceName)
{
	Monitor::Enter(ProtectEnvironments);
	NvDotnetEnv^ env = nullptr;
	if(Environments->Contains(ServiceName))
		env = (NvDotnetEnv^)Environments[ServiceName];
	Monitor::Exit(ProtectEnvironments);
	return env;
}

// Remove all items from the list
void NvDotnetEnvList::RemoveAll()
{
	Clear();
}

// Clear all environments and reset the hash table
void NvDotnetEnvList::Clear()
{
	Monitor::Enter(ProtectEnvironments);
	IDictionaryEnumerator^ Enumerator = Environments->GetEnumerator();
	while (Enumerator->MoveNext())
	{
		NvDotnetEnv^ env = (NvDotnetEnv^)(Enumerator->Value);
		if(env != nullptr)
			delete env;
	}
	Environments->Clear();
	Monitor::Exit(ProtectEnvironments);
}




// Library exported functions

int NvInitDotnet(const char* NirvaDir)
{
	sNirvaDir = NirvaDir;
	//remove the last "\" character;
	sNirvaDir = sNirvaDir.substr(0, sNirvaDir.length()-1);
	return 1;
}


void NvFreeDotnet()
{
	// Clear all domains
	Globals::Domains.Clear();
}

// NvFreeComponent is called by Nirva when a service, application, web service or system is closed
// Type is a domain type (see edotnet.h) and Name is the Type Name (name of service or application for example)
void NvDotnetUnloadDomain(const char* Type, const char* Name)
{
	String^ TypeHere = gcnew String(Type);
	String^ NameHere = gcnew String(Name);
	Globals::Domains.Remove(TypeHere, NameHere);
}


// Run a .net class
// ClassName has the form TYPE::NAME::Assembly:class
// TYPE is one of APPLICATION, SERVICE, WSERVICE or SYSTEM
// NAME is the Application, service or web service name in uppercase ("SYSTEM" si type =SYSTEM)
// Assembly is the complete path of the assembly file (procedure file)
// Class is optional and gives the class name inside the assembly (ex ProcNet.Class1 (if namespace is used), or ProcNet)
// If Class is not given, we use the name of the assembly file as class name
int NvRunDotnetClass(const char* ClassName, void *pCommand)
{
	if(ClassName == NULL)
		return 0;
	if(pCommand == NULL)
		return 0;

	ProcedureEntryPoint^ obj = nullptr;	// Interface  for procedure entry point

	string sClassName = ClassName;
	string sType = "";
	string sName = "";
	string sAssembly = "";
	string sAssemblyName = "";
	string sClass = "";

	string::size_type index = sClassName.find("::");
	if(index == string::npos)
		return 0;
	sType = sClassName.substr(0, index);
	sClassName = sClassName.substr(index+2);

	index = sClassName.find("::");
	if(index == string::npos)
		return 0;
	sName = sClassName.substr(0, index);
	sAssembly = sClassName.substr(index+2);

	// replace all \ with /
	while((index = sAssembly.find('\\')) != string::npos)
		sAssembly.replace(index, 1, 1, '/');
	index = sAssembly.rfind("/");
	if(index != string::npos) // Should always occur
		sAssemblyName = sAssembly.substr(index+1);

	index = sAssemblyName.rfind(":");
	if(index != string::npos)
	{
		sClass = sAssemblyName.substr(index+1);
		sAssemblyName = sAssemblyName.substr(0, index);
		index = sAssembly.rfind(":");
		sAssembly = sAssembly.substr(0, index);
	}
	index = sAssemblyName.rfind(".");
	if(index != string::npos)
		sAssemblyName = sAssemblyName.substr(0, index);
	if(!sClass.length())
		sClass = sAssemblyName;

	String^ Type = gcnew String(sType.c_str());
	String^ Name = gcnew String(sName.c_str());

	String^ AssemblyPath = gcnew String(sAssembly.c_str());
	String^ ClassNameHere = gcnew String(sClass.c_str());
	

	// Domain init
	AppDomain^ domain = Globals::Domains.GetAdd(Type, Name);
	if(domain == nullptr)
		return 0;

	// get an instance of the class
	try
	{
		obj = (ProcedureEntryPoint^) domain->CreateInstanceFromAndUnwrap(AssemblyPath, ClassNameHere);
	}
	catch(Exception^ e)
	{
		Console::Write("Error CreateInstanceFromAndUnwrap: ");
		Console::WriteLine(e->Message);
		return 0;
	}

	NvsInt^ nvsint = gcnew NvsInt(pCommand);
	// Run the class method entry point
	try
	{
		obj->Main(nvsint);
	}
	catch(Exception^ e)
	{
		Console::WriteLine("Error Hello");
		Console::WriteLine(e->Message);
		delete nvsint;	// This calls the destructor of nvsint (similar to IDispose interface in C#)
		delete obj;
		return 0;
	}
	delete nvsint; // This calls the destructor of nvsint (similar to IDispose interface in C#)
	delete obj;
	return 1;
}


// Main Nirva external service entry point
int NvsCommand(void *pCommand)
{
	// The pCommand points in fact on a NvServiceCommand class
	NvServiceCommand *Command = (NvServiceCommand *)pCommand;
	int RetVal = 0;
	// We get the service name
	int BufferSize;
	Command->GetParameter("NV_DOTNET_SERVICE_NAME", NULL, 0, &BufferSize);
	BufferSize += 10;
	char *Buffer = (char *)malloc(BufferSize);
	Command->GetParameter("NV_DOTNET_SERVICE_NAME", Buffer, BufferSize);
	string ServiceName = Buffer;
	free(Buffer);


	// Check the service init and exit commands
	if(Command->IsCommand("SYSTEM", "NV_INIT_SERVICE", NV_SOURCE_SERVER))
		return NvInitService(Command, ServiceName);


	if(Command->IsCommand("SYSTEM", "NV_EXIT_SERVICE", NV_SOURCE_SERVER))
		return NvExitService(Command, ServiceName);

	// Normal command
	RetVal = NvCommand(Command, ServiceName);

	return RetVal;
}



// Called one time when the service is first acceded
int NvInitService(NvServiceCommand *Command, string& ServiceName)
{
	char Buffer[1024];
	Command->GetParameter("NV_DOTNET_SERVICE_PATH", Buffer, sizeof(Buffer));
	string ServicePath = Buffer;
	if(!ServicePath.length())
		return 0;

	// The service path is the complete assembly path eventually followed by :classname.

	string sAssemblyPath = ServicePath;
	string sAssemblyName = "";
	string sClass = "";

	// replace all \ with /
	string::size_type index;
	while((index = sAssemblyPath.find('\\')) != string::npos)
		sAssemblyPath.replace(index, 1, 1, '/');
	index = sAssemblyPath.rfind("/");
	if(index != string::npos) // Should always occur
		sAssemblyName = sAssemblyPath.substr(index+1);

	index = sAssemblyName.rfind(":");
	if(index != string::npos)
	{
		sClass = sAssemblyName.substr(index+1);
		sAssemblyName = sAssemblyName.substr(0, index);
		index = sAssemblyPath.rfind(":");
		sAssemblyPath = sAssemblyPath.substr(0, index);
	}
	
	index = sAssemblyName.rfind(".");
	if(index != string::npos)
		sAssemblyName = sAssemblyName.substr(0, index);
	if(!sClass.length())
		sClass = sAssemblyName;

	String^ sAssemblyPathHere = gcnew String(sAssemblyPath.c_str());
	String^ sClassHere = gcnew String(sClass.c_str());

	String^ ServiceNameHere = gcnew String(ServiceName.c_str());
	NvDotnetEnv^ DotnetEnv = Globals::Environments.GetDotnetEnv(ServiceNameHere);
	if(DotnetEnv != nullptr)
		return 1;	// ever initalized

	DotnetEnv = gcnew NvDotnetEnv;
	DotnetEnv->ServiceName = ServiceNameHere;

	if(!DotnetEnv->LoadClass(sAssemblyPathHere, sClassHere))
	{
		delete DotnetEnv;
		// Remove the service domain
		Globals::Domains.Remove("SERVICE", ServiceNameHere);
		return 0;
	}

	Globals::Environments.AddDotnetEnv(ServiceNameHere, DotnetEnv);

	int RetVal = 1;
	NvCmd^ nvcmd = gcnew NvCmd(Command, ServiceNameHere);
	// Run the class method entry point
	try
	{
		if(!DotnetEnv->obj->Init(nvcmd))
			RetVal = 0;
	}
	catch(Exception^ e)
	{
		Console::WriteLine(e->Message);
		RetVal = 0;
	}
	delete nvcmd; // This calls the destructor of nvsint (similar to IDispose interface in C#)
	if(!RetVal)
	{
		Globals::Environments.RemoveDotnetEnv(ServiceNameHere);
		// Remove the service domain
		Globals::Domains.Remove("SERVICE", ServiceNameHere);
	}
	return RetVal;
}


// Called one time when the service is closed
int NvExitService(NvServiceCommand *Command, string& ServiceName)
{
	String^ ServiceNameHere = gcnew String(ServiceName.c_str());
	NvDotnetEnv^ DotnetEnv =  Globals::Environments.GetDotnetEnv(ServiceNameHere);
	if(DotnetEnv == nullptr)
		return 0;	// not initialized, should not occur


	int RetVal = 1;
	NvCmd^ nvcmd = gcnew NvCmd(Command, ServiceNameHere);
	// Run the class method entry point
	try
	{
		if(!DotnetEnv->obj->Exit(nvcmd))
			RetVal = 0;
	}
	catch(Exception^ e)
	{
		Console::WriteLine(e->Message);
		RetVal = 0;
	}
	delete nvcmd; // This calls the destructor of nvsint (similar to IDispose interface in C#)
	Globals::Environments.RemoveDotnetEnv(ServiceNameHere);

	return RetVal;
}



// Renderer entry point
int NvsRender(void *RenderInterface, const char *ServiceName)
{
	// The RenderInterface points in fact on a NvRendererInt class
	NvRendererInt *Render = (NvRendererInt*)RenderInterface;
	int RetVal = 0;
	string ServiceNameHere = ServiceName;

	// Do the job
	RetVal = NvRender(Render, ServiceNameHere);

	return RetVal;
}


// Called for each service command
int NvCommand(NvServiceCommand *Command, string& ServiceName)
{
	String^ ServiceNameHere = gcnew String(ServiceName.c_str());
	NvDotnetEnv^ DotnetEnv =  Globals::Environments.GetDotnetEnv(ServiceNameHere);

	if(DotnetEnv == nullptr)
		return 0;	// not initialized, should not occur

	int RetVal = 1;
	NvCmd^ nvcmd = gcnew NvCmd(Command, ServiceNameHere);
	// Run the class method entry point
	try
	{
		if(!DotnetEnv->obj->Command(nvcmd))
			RetVal = 0;
	}
	catch(Exception^ e)
	{
		Console::WriteLine(e->Message);
		RetVal = 0;
	}
	delete nvcmd; // This calls the destructor of nvsint (similar to IDispose interface in C#)

	return RetVal;
}


// Called for each service command
int NvRender(NvRendererInt *Render, string& ServiceName)
{
	String^ ServiceNameHere = gcnew String(ServiceName.c_str());
	NvDotnetEnv^ DotnetEnv =  Globals::Environments.GetDotnetEnv(ServiceNameHere);

	if(DotnetEnv == nullptr)
	{
		Render->SetReason("Dotnet service not initialized");
		return 0;	// not initialized, should not occur
	}

	if(DotnetEnv->obj->GetType()->GetMethod("Render") == nullptr)
	{
		Render->SetReason("This is not a renderer service");
		return 0;	// not initialized, should not occur
	}

	int RetVal = 1;
	bool ExceptionOccurred = false;

	// PM TODO Create an instance of the NvRend class (derived from nvrend defined in nirvadn.dll)
	Nirva::nvrend^ Mynvrend = gcnew Nirva::nvrend();
	Mynvrend->AppName = StringConvert::NewStringUTF(Render->GetAppName());
	Mynvrend->PageName = StringConvert::NewStringUTF(Render->GetPageName());
	Mynvrend->SessionId = StringConvert::NewStringUTF(Render->GetSessionId());
	Mynvrend->InputEncoding = StringConvert::NewStringUTF(Render->GetInputEncoding());
	Mynvrend->InputLength = Render->GetInputDataSize();
	Mynvrend->Input = gcnew array<Byte>(Render->GetInputDataSize());
	try
	{
		Marshal::Copy((IntPtr)((char *)(Render->GetInputData())), Mynvrend->Input, 0, Render->GetInputDataSize());
	}
	catch(Exception^ e)
	{
		Render->SetReason("Error when copying input buffer");
		RetVal = 0;
	}

	Render->FreeInputData();

	if(!RetVal)
	{
		delete Mynvrend;
		return 0;
	}

	// Run the Render method entry point
	try
	{
		if(!DotnetEnv->obj->Render(Mynvrend))
			RetVal = 0;
	}
	catch(Exception^ e)
	{
		Console::WriteLine(e->Message);
		Render->SetReason("Exception occured in Render method");
		RetVal = 0;
		ExceptionOccurred = true;
	}
	// Get the reason
	if(!RetVal)
	{
		LPSTR str = StringConvert::GetStringUTFChars(Mynvrend->Reason);
		Render->SetReason(str);
		StringConvert::ReleaseStringUTFChars(str);
	}

	// Get the content-type
	if(RetVal)
	{
		LPSTR str = StringConvert::GetStringUTFChars(Mynvrend->ContentType);
		Render->SetContentType(str);
		StringConvert::ReleaseStringUTFChars(str);
	}
	
	// Get the output stream
	if(RetVal)
	{
		// Get the result buffer
		if(Mynvrend->Output != nullptr)
		{
			unsigned long OutputLength = Mynvrend->Output->Length;
			if(OutputLength)
			{
				char *Buffer = Render->AllocOutputBuffer(OutputLength);
				if(Buffer != NULL)
				{
					try
					{
						Marshal::Copy(Mynvrend->Output, 0, (IntPtr)Buffer, OutputLength);
					}
					catch(Exception^ e)
					{
						Render->SetReason("Error when copying output buffer");
						RetVal = 0;
					}
				}
				else
				{
					Render->SetReason("Not enough memory to allocate output buffer");
				}
			}
		}
	}


	delete Mynvrend; // This calls the destructor of nvrend (similar to IDispose interface in C#)

	return RetVal;
}
