// ***************************************************************************************
// File: garbage.h
// Description: file garbage collector declarations
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifndef _GARBAGE_H
#define _GARBAGE_H


// Classes declared in this file
class NvGarbageEntry;
class NvGarbage;

#include "task.h"
#include <vector>
#include <string>



/////////////////////////////////////////////////////////////////////////////
// NvGarbageEntry

class NvGarbageEntry
{
public:
	std::string m_sFileName;				// Filename
	int m_iMaxSeconds;					// Max number of seconds for file to stay
	time_t m_InitialTime;					// Initial time for file

	
	NvGarbageEntry();
	virtual ~NvGarbageEntry();
};


class NvGarbage : public NvObject
{
public:
	std::vector<void *> Entries;		// Entries

	
	NvGarbage();
	virtual ~NvGarbage();

	// Internal function
	NvGarbageEntry* GetEntry(std::string FileName);
	int GetEntryIndex(std::string FileName);
	
	void Clear();
	void AddEntry(std::string FileName, int MaxSecond = 0);
	void RemoveEntry(std::string FileName);
	void RemoveExpired();
	bool IsInGarbage(std::string FileName);
};


#endif //_GARBAGE_H
