// ***************************************************************************************
// File: nidget.h
// Description: declarations of the Nirva nidget framework
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifndef _NIDGET_H
#define _NIDGET_H


// Classes declared in this file
class NvNidgetDoc;
class NvNidget;
class NvNidgetView;
class NvNidgetFramework;

#include "task.h"
#include "util.h"
#include <string>
#include <vector>
#include <map>


// document type
#define NV_NIDGET_DOC_TYPE_DYNAMIC			0
#define NV_NIDGET_DOC_TYPE_STATIC			1
#define NV_NIDGET_DOC_TYPE_INLINE			2

// view type
#define NV_NIDGET_VIEW_TYPE_DYNAMIC			0
#define NV_NIDGET_VIEW_TYPE_STATIC			1
#define NV_NIDGET_VIEW_TYPE_HTML			2

// view type2
#define NV_NIDGET_VIEW_TYPE_NORMAL			0
#define NV_NIDGET_VIEW_TYPE_HEADER			1
#define NV_NIDGET_VIEW_TYPE_FOOTER			2


/////////////////////////////////////////////////////////////////////////////
// NvNidgetDoc class
class NvNidgetDoc : public NvObject
{
public:
	std::string m_sName;		// Doc name
	int m_iType;				// Doc type
	std::string m_sLoad;		// Load document command (Used at view level)

	NvNidgetDoc();
	virtual ~NvNidgetDoc();

	void Clear(bool All=true);
};


/////////////////////////////////////////////////////////////////////////////
// NvNidget class
class NvNidget : public NvObject
{
public:
	std::string m_sName;				// Nidget name
	std::string m_sClass;				// Nidget class name
	std::vector<std::string> m_vDocs;	// Documents used by the nidget

	NvNidget();
	virtual ~NvNidget();

	void Clear(bool All=true);
};


/////////////////////////////////////////////////////////////////////////////
// NvNidgetView class
class NvNidgetView : public NvObject
{
public:
	std::string m_sName;								// View name (in fact the file name)
	int m_iType;										// View type (STATIC or DYNAMIC)
	int m_iType2;										// View type (HEADER, NORMAL or FOOTER)
	std::string m_sClass;								// View css class
	std::string m_sParameters;							// View parameters file name
	std::string m_sLabels;								// View parameters labels file name
	std::string m_sTitle;								// View title
	std::string m_sLanguage;							// Default language for static views
	std::map<std::string,std::string> m_mInfo;			// View info section
	std::map<std::string,NvNidgetDoc*> m_mDocs;			// Documents
	std::string m_sViewData;							// View data (before resolving session ID)
	int SessionIdIndex;									// Index to insert session id in m_sViewData
	int LanguageIndex;									// Index to insert language in m_sViewData
	std::map<std::string,std::string> m_mParameters;	// File parameters already built
	std::string m_sSectionsData;						// Sections data
	std::map<std::string,NvNidget*> m_mNidgets;			// View Nidgets
	std::string m_sHeader;								// Header
	std::string m_sFooter;								// Footer
	std::vector<std::string> m_vSections;				// Section names (used only when building the view)
	std::map<std::string,std::string> m_mForms;			// Form names (used only when building the view) 

	NvNidgetView();
	virtual ~NvNidgetView();

	void Clear(bool All=true);
};


/////////////////////////////////////////////////////////////////////////////
// NvNidgetFramework class
class NvNidgetFramework : public NvObject
{
public:
	std::string m_sAppName;								// Application name
	std::string m_sFrameworkSourceDir;					// Source directory
	std::map<std::string,std::string> m_mInfo;			// Framework info section
	std::map<std::string,std::string> m_mPackages;		// Nidget packages
	std::map<std::string,std::string> m_mIncludes;		// Global includes
	std::map<std::string,NvNidgetView*> m_mViews;		// Views
	std::string m_sChecksum;							// Used for checking modifications
	std::string m_sDebugLevel;							// Debug level
	std::string m_sErrorView;							// Error View
	std::map<std::string,std::string> m_mHeaders;		// Header views
	std::map<std::string,std::string> m_mFooters;		// Footer views
	std::string m_sIcon;								// Application icon
	std::string m_sTheme;								// Theme (css file)

	NvNidgetFramework();
	virtual ~NvNidgetFramework();

	void Clear();

	// Check and publish the framework if necessary
	bool CheckPublish(NvCommand *CommandToExecute);

	// Get the view data (main framework entry point)
	bool GetView(const char* Name, const char* SessionId, NvCommand *CommandToExecute, char** Result, unsigned long* ResultSize, bool ForceCheck = false, bool ErrorView = false);
	// Check if the view is static or not
	bool IsStaticView(const char* Name, bool ForceCheck = false);
	// Load the view
	bool LoadView(NvNidgetView* View, NvCommand *CommandToExecute, bool bDoPublish);
	// Check if the parameters have to be built
	bool CheckViewParameters(std::string& ViewName, std::string& Language);
	// Build the view param file and publishes it
	bool BuildViewParameters(NvNidgetView* View, NvCommand *CommandToExecute, std::string Language);
	// Get the framework source dir
	std::string& GetFrameworkSourceDir();
	// Create the framework skeleton
	bool Skeleton();
};




/////////////////////////////////////////////////////////////////////////////
// NvJspCleaner class
class NvJspCleaner : public NvObject
{
public:
	int   theA;
	int   theB;
	int   theLookahead;
	FILE *FileSource;
	FILE *FileDest;

	NvJspCleaner();
	virtual ~NvJspCleaner();

	int isAlphanum(int c);
	int get();
	int peek();
	int next();
	bool action(int d);
	bool jsmin();
	bool DoClean(const char* SrcFile, const char* DstFile);
};


#endif //_NIDGET_H
