// ***************************************************************************************
// File: test.h
// Description: declarations of the Nirva test classes
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifndef _TEST_H
#define _TEST_H


// Classes declared in this file
class NvTest;
class NvTestSet;
class NvTestSetList;

#include "task.h"
#include "util.h"
#include <string>
#include <map>


// test type
#define NV_TEST_PROC			0
#define NV_TEST_URL				1


/////////////////////////////////////////////////////////////////////////////
// NvTest class
class NvTest : public NvObject
{
public:
		
	std::string m_sName;						// Test name
	std::string m_sDesc;						// Test description
	std::string m_sProc;						// Test procedure
	std::string m_sUrl;							// Test url
	int m_iType;								// Test type
	std::map<std::string,std::string>	m_mParams;	// Parameters

	NvTest();
	virtual ~NvTest();
};


/////////////////////////////////////////////////////////////////////////////
// NvTestSet class
class NvTestSet : public NvObject
{
public:
		
	std::string m_sName;						// Set name
	std::string m_sDesc;						// Set description
	std::map<std::string,NvTest*> m_mTests;		// Test list

	NvTestSet();
	virtual ~NvTestSet();

	// Removes all tests from the list
	void Clear();
	// Get a specific test
	NvTest* GetTest(const char *Name);
	// Load the set from file
	bool Load(std::string& FileName);
	// Load the optional user parameter file
	bool LoadTestParameters(std::string& FileName);
};


/////////////////////////////////////////////////////////////////////////////
// NvTestSetList class
class NvTestSetList : public NvObject
{
private:
	NvTestSet* InternalGetSet(const char *Name);

public:
	std::map<std::string,NvTestSet*> m_mTestSets;	// Test set list
	std::string m_sDirPath;							// Test set directory (where test sets xml files reside)
	
	NvTestSetList();
	virtual ~NvTestSetList();

	// Removes all sets from the list
	void Clear();

	// Load the set from directory
	bool Load();

	// Get set info
	bool GetInfo(const char *SetName, NvObjTable *SetInfo);

	// Get test info
	bool GetTestInfo(const char *SetName, const char *TestName, NvObjTable *TestInfo);

};


#endif //_TEST_H
