// ***************************************************************************************
// File: util.cpp
// Description: utility functions and classes
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifdef WIN32
#pragma warning( disable : 4996 )
#endif

#ifdef NV_SERVER
#include "nvs.h"
#else
#include "../nvc/nvcc.h"
#endif
#include "utf8.h"

#ifdef NV_SERVER
	#ifdef WIN32
		#ifdef _DEBUG
			#include <stdlib.h>
			#include <crtdbg.h>
			#include "psapi.h"
			#define NVNEW new( _NORMAL_BLOCK, __FILE__, __LINE__)
		#else // _DEBUG
			#define NVNEW new
		#endif // _DEBUG
	#else	//WIN32
		#define NVNEW new
	#endif	//WIN32
#else	//NV_SERVER
	#define NVNEW new
#endif	//NV_SERVER


#ifndef WIN32
extern char **environ;
int my_system (const char *command, unsigned int TimeOut = 0);
#endif


NvMutex	ProtectTempFile;
int TempFileIndex = 0;

#ifdef NV_SERVER
extern NvMutex ProtectConsoleLog;				// protect console output defeined in nvs.cpp
#endif	//NV_SERVER

using namespace std;







//**********************************************************************************
//**********************************************************************************
// These very usefull function unfortunatly doesn't exist on unix, so we do them
//**********************************************************************************
//**********************************************************************************
#ifndef WIN32
char *strlwr(char *string)
{
	for(int i=0; i<strlen(string); i++)
	{
		if(isascii(string[i]) && isupper(string[i]))
			string[i] = (char)(tolower(string[i]));
	}
	return string;
}
char *strupr(char *string)
{
	for(int i=0; i<strlen(string); i++)
	{
		if(isascii(string[i]) && islower(string[i]))
			string[i] = (char)(toupper(string[i]));
	}
	return string;
}
#endif


int CopyFileEx(const char *SourceFile, const char *DestFile, NvInt64 Offset, NvInt64 BytesToCopy)
{
	FILE *FileSource;
	FILE *FileDest;
	unsigned int ReadBytes;
	unsigned short error;
	char *buffer;
	
	
	if(SourceFile == NULL)
		return 0;
	if(!strlen(SourceFile))
		return 0;
	
	if(DestFile == NULL)
		return 0;
	if(!strlen(DestFile))
		return 0;
	
	buffer = NVNEW char[16384];

	if(buffer == NULL)
		return 0;

	FileSource = Nvfopen(SourceFile, "rb");
	if(FileSource == NULL)
	{
		delete [] buffer;
		return 0;
	}

	FileDest = Nvfopen(DestFile, "wb");
	if(FileDest == NULL)
	{
		fclose(FileSource);
		delete [] buffer;
		return 0;
	}

	error = 0;
	
	
	// we get the result file size
	Nvfseek(FileSource, 0, SEEK_END);
	NvInt64 Length = Nvftell(FileSource);

	if(Nvfseek(FileSource, Offset, SEEK_SET) != 0)
	{
		fclose(FileSource);
		fclose(FileDest);
		delete [] buffer;
		return 0;
	}


	if(BytesToCopy == -1)
		BytesToCopy = Length - Offset;
	else
		if(BytesToCopy > (Length - Offset))
			BytesToCopy = Length - Offset;

	unsigned int NbLoops = (unsigned int)(BytesToCopy / 16384);
	unsigned int LastBytes = (unsigned int)(BytesToCopy % 16384);

	for (unsigned int i=0; i<NbLoops; i++)
	{
		ReadBytes = (unsigned int)fread(buffer, 1, 16384, FileSource);
		if(ReadBytes != 16384)
		{
			fclose(FileSource);
			fclose(FileDest);
			delete [] buffer;
			return 0;
		}
		if(fwrite(buffer, 1, ReadBytes, FileDest)  != ReadBytes)
		{
			fclose(FileSource);
			fclose(FileDest);
			delete [] buffer;
			return 0;
		}

	}
	if(LastBytes)
	{
		ReadBytes = (unsigned int)fread(buffer, 1, LastBytes, FileSource);
		if(ReadBytes != LastBytes)
		{
			fclose(FileSource);
			fclose(FileDest);
			delete [] buffer;
			return 0;
		}
		if(fwrite(buffer, 1, ReadBytes, FileDest)  != ReadBytes)
		{
			fclose(FileSource);
			fclose(FileDest);
			delete [] buffer;
			return 0;
		}
	}


	fclose(FileSource);
	fclose(FileDest);
	delete [] buffer;
	
	return 1;
}


int CopyFileEx(FILE *FileSource, FILE *FileDest, NvInt64 Offset, NvInt64 BytesToCopy)
{
	unsigned int ReadBytes;
	unsigned short error;
	char *buffer;
	
	buffer = NVNEW char[16384];

	if(buffer == NULL)
		return 0;

	error = 0;
	
	// we get the result file size
	Nvfseek(FileSource, 0, SEEK_END);
	NvInt64 Length = Nvftell(FileSource);

	if(Nvfseek(FileSource, Offset, SEEK_SET) != 0)
	{
		delete [] buffer;
		return 0;
	}


	if(BytesToCopy == -1)
		BytesToCopy = Length - Offset;
	else
		if(BytesToCopy > (Length - Offset))
			BytesToCopy = Length - Offset;

	unsigned int NbLoops = (unsigned int)(BytesToCopy / 16384);
	unsigned int LastBytes = (unsigned int)(BytesToCopy % 16384);

	for (unsigned int i=0; i<NbLoops; i++)
	{
		ReadBytes = (unsigned int)fread(buffer, 1, 16384, FileSource);
		if(ReadBytes != 16384)
		{
			delete [] buffer;
			return 0;
		}
		if(fwrite(buffer, 1, ReadBytes, FileDest)  != ReadBytes)
		{
			delete [] buffer;
			return 0;
		}

	}
	if(LastBytes)
	{
		ReadBytes = (unsigned int)fread(buffer, 1, LastBytes, FileSource);
		if(ReadBytes != LastBytes)
		{
			delete [] buffer;
			return 0;
		}
		if(fwrite(buffer, 1, ReadBytes, FileDest)  != ReadBytes)
		{
			delete [] buffer;
			return 0;
		}
	}


	delete [] buffer;
	
	return 1;
}


int AppendFileEx(const char *SourceFile, const char *DestFile, NvInt64 Offset, NvInt64 BytesToCopy)
{
	FILE *FileSource;
	FILE *FileDest;
	
	if(SourceFile == NULL)
		return 0;
	if(!strlen(SourceFile))
		return 0;
	
	if(DestFile == NULL)
		return 0;
	if(!strlen(DestFile))
		return 0;
	

	FileSource = Nvfopen(SourceFile, "rb");
	if(FileSource == NULL)
		return 0;

	FileDest = Nvfopen(DestFile, "a+b");
	if(FileDest == NULL)
	{
		fclose(FileSource);
		return 0;
	}

	int RetVal = AppendFileEx(FileSource, FileDest, Offset, BytesToCopy);

	fclose(FileSource);
	fclose(FileDest);

	return RetVal;
}

NvInt64 GetFileLength(const char* FileName)
{
	FILE *File = Nvfopen(FileName, "rb");
	if(File == NULL)
		return 0;

	
	// we get the result file size
	Nvfseek(File, 0, SEEK_END);
	NvInt64 Length = Nvftell(File);

	fclose(File);
	return Length;
}

NvInt64 GetFileLength(FILE *File)
{
	NvInt64 ActualPosition = Nvftell(File);
	
	// we get the result file size
	Nvfseek(File, 0, SEEK_END);
	NvInt64 Length = Nvftell(File);

	Nvfseek(File, ActualPosition, SEEK_SET);

	return Length;
}


int AppendFileEx(FILE *FileSource, FILE *FileDest, NvInt64 Offset, NvInt64 BytesToCopy)
{
	unsigned int ReadBytes;
	unsigned short error;
	char *buffer;
	
	
	if(FileSource == NULL)
		return 0;
	
	if(FileDest == NULL)
		return 0;
	
	buffer = NVNEW char[16384];

	if(buffer == NULL)
		return 0;


	error = 0;
	
	
	// we get the result file size
	Nvfseek(FileSource, 0, SEEK_END);
	NvInt64 Length = Nvftell(FileSource);

	if(Nvfseek(FileSource, Offset, SEEK_SET) != 0)
	{
		delete [] buffer;
		return 0;
	}


	if(BytesToCopy == -1)
		BytesToCopy = Length - Offset;
	else
		if(BytesToCopy > (Length - Offset))
			BytesToCopy = Length - Offset;

	unsigned int NbLoops = (unsigned int)(BytesToCopy / 16384);
	unsigned int LastBytes = (unsigned int)(BytesToCopy % 16384);

	for (unsigned int i=0; i<NbLoops; i++)
	{
		ReadBytes = (unsigned int)fread(buffer, 1, 16384, FileSource);
		if(ReadBytes != 16384)
		{
			delete [] buffer;
			return 0;
		}
		if(fwrite(buffer, 1, ReadBytes, FileDest)  != ReadBytes)
		{
			delete [] buffer;
			return 0;
		}

	}
	if(LastBytes)
	{
		ReadBytes = (unsigned int)fread(buffer, 1, LastBytes, FileSource);
		if(ReadBytes != LastBytes)
		{
			delete [] buffer;
			return 0;
		}
		if(fwrite(buffer, 1, ReadBytes, FileDest)  != ReadBytes)
		{
			delete [] buffer;
			return 0;
		}
	}


	delete [] buffer;
	
	return 1;
}

//**********************************************************************************
//**********************************************************************************
// This function creates a unique temporary file
// It is protected by a mutex in order to be shure that the file is unique
//**********************************************************************************
//**********************************************************************************
bool GetTemporaryFile(string &FileName, string Dir, string Extension, string Prefix, string Suffix)
{
	CleanString(Dir);
	CleanString(Extension);
	CleanString(Prefix);
	CleanString(Suffix);

	bool ret = true;

	NV_LOCKLOCK(&ProtectTempFile);

	AddDirSuffix(Dir);
	
	FileName = "";
	char *FileNameHere;
	int FileNameMaxSize = (int)(Dir.length() + Extension.length() + Prefix.length() + Suffix.length() + 512);
	FileNameHere = (char *)malloc(FileNameMaxSize);
	if(FileNameHere == NULL)
		return false;
	
	// We generate a file name until the file doesn't exist and can be opened in write mode
	unsigned int nbloop = 0;
	while(1)
	{
		sprintf(FileNameHere, "%s%s%08x%04x%04x%s", Dir.c_str(), Prefix.c_str(), TempFileIndex, getpid(), rand(), Suffix.c_str());
		if(Extension.length())
		{
			if(Extension[0] == '.')
				Extension.erase(0, 1);
			if(Extension.length())
			{
				strcat(FileNameHere, ".");
				strcat(FileNameHere, Extension.c_str());
			}
		}
		FILE * File = Nvfopen(FileNameHere, "r");
		if(File == NULL)	// the file doesn't exist
		{
			File = Nvfopen(FileNameHere, "w");	// so we try to open it for writting in order to make a zero length file
			if(File == NULL)				// if error we continue with another file name but no more than 10 times
			{								// because we may have a problem of write permission on a directory
				if(nbloop < 10)
				{
					nbloop++;
					TempFileIndex++;
					continue;
				}
				else
				{
					ret = false;
					break;
				}
			}
			
			fclose(File);					// otherwise we close the file and stop the loop (File is really correct and unique)
			break;
		}
		fclose(File);
		TempFileIndex++;
	}

	TempFileIndex++;
	if(ret)
		FileName = FileNameHere;

	free(FileNameHere);
	NV_UNLOCKLOCK(&ProtectTempFile);
	return ret;
}


void RemoveFile(const char *FileName)
{
	if(FileName == NULL)
		return;
	if(!strlen(FileName))
		return;
	unlink(FileName);
}


int FileExist(const char *FileName)
{
	if(FileName == NULL)
		return 0;
	if(!strlen(FileName))
		return 0;
	FILE *stream = Nvfopen(FileName, "rb");
	if(stream == NULL)
		return 0;
	fclose(stream);
	return 1;
}


int DirExist(const char *DirName)
{
	if(DirName == NULL)
		return 0;

	char DirNameHere[NV_MAX_PATHLENGTH];
	strncpy(DirNameHere, DirName, NV_MAX_PATHLENGTH-1);
	if(strlen(DirNameHere))			// we remove eventual last slash
		if(DirNameHere[strlen(DirNameHere)-1] == '/')
			DirNameHere[strlen(DirNameHere)-1] = '\0';
	if(strlen(DirNameHere))			// we remove eventual last backslash
		if(DirNameHere[strlen(DirNameHere)-1] == '\\')
			DirNameHere[strlen(DirNameHere)-1] = '\0';
	if(!strlen(DirNameHere))
		return 0;
#ifdef WIN32
	// We use functions for finding directories given a path
	WIN32_FIND_DATA FindData;
	// Try to find the first file
	HANDLE hFileFind = FindFirstFile(DirNameHere, &FindData);
	if(hFileFind == INVALID_HANDLE_VALUE)
		return 0;	// No file
	if((FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0)
	{
		// this is a file
		FindClose(hFileFind);	// we close the finder
		return 0;	// Not a directory
	}
	FindClose(hFileFind);	// we close the finder
#else

	// for unix, we try to open the directory file

	// open directory file
	DIR *Dir = opendir(DirNameHere);
	if(Dir == NULL)
		return 0;		// cannot open directory file
	closedir(Dir);	// close the directory file
#endif

	return 1;
}


bool DirList(const char *DirName, const char *Root, vector<string> *List)
{
	if(DirName == NULL)
		return false;

	char DirNameHere[NV_MAX_PATHLENGTH];
	strncpy(DirNameHere, DirName, NV_MAX_PATHLENGTH-1);

	char FilesToFind[NV_MAX_PATHLENGTH];

#ifdef WIN32
	strncpy(FilesToFind, DirNameHere, NV_MAX_PATHLENGTH-1);
	if(strlen(FilesToFind))
	{
		// We add the last \ character
		if(FilesToFind[strlen(FilesToFind) - 1] != '\\')
			strcat(FilesToFind, "\\");
	}
	if(Root == NULL)
		strcat(FilesToFind, "*");
	else
		strcat(FilesToFind, Root);
	WIN32_FIND_DATA FindData;
	// Try to find the first file
	HANDLE hFileFind = FindFirstFile(FilesToFind, &FindData);
	if(hFileFind == INVALID_HANDLE_VALUE)
		return true;	// No file
	while(1)
	{
		// We remove all the found files
		if((FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0)
		{
			if(FindData.cFileName[0] != '.')
			{
				string FoundDirName = FindData.cFileName;
				List->push_back(FoundDirName);	// This directory is a good candidate
			}
		}

		if(!FindNextFile(hFileFind, &FindData))
			break;	// no more file

	}
	FindClose(hFileFind);	// we close the finder
#else

	string RootHere = "";
	if(Root != NULL)
		RootHere = Root;
	if(RootHere == "*")
		RootHere = "";
	char WorkDirHere[NV_MAX_PATHLENGTH];
	strncpy(WorkDirHere, DirNameHere, NV_MAX_PATHLENGTH-1);
	if(strlen(WorkDirHere))
	{
		// We remove the last / character
		if(WorkDirHere[strlen(WorkDirHere) - 1] == '/')
			WorkDirHere[strlen(WorkDirHere) - 1] = '\0';
	}
	else
		strcpy(WorkDirHere, ".");	// we are in the current directory

	// open directory file
	DIR *Dir = opendir(WorkDirHere);
	if(Dir == NULL)
		return true;		// cannot open directory file
	
	dirent *DirEntry;
	while((DirEntry = readdir(Dir)) != 0)
	{
		if(DirEntry->d_name[0] == '.')
			continue;
		strncpy(FilesToFind, WorkDirHere, NV_MAX_PATHLENGTH-1);
		if(strlen(FilesToFind))
		{
			// We add the last \ character
			if(FilesToFind[strlen(FilesToFind) - 1] != '/')
				strcat(FilesToFind, "/");
		}
		strcat(FilesToFind, DirEntry->d_name);
		DIR *DirHere = NULL;
		if((DirHere = opendir(FilesToFind)) == NULL)
			continue;
		closedir(DirHere);
		// We have found a real subdirectory
		// The subdirectory name seems to be OK
		string FoundDirName = DirEntry->d_name;
		if(RootHere.length())
			if(strstr(DirEntry->d_name, RootHere.c_str()) != DirEntry->d_name)
				continue;
		List->push_back(FoundDirName);	// This directory is a good candidate
	}
	closedir(Dir);	// close the directory file
#endif

	return true;
}


bool IsDirEmpty(const char *DirName)
{
	if(DirName == NULL)
		return true;

	bool IsEmpty = true;

	char DirNameHere[NV_MAX_PATHLENGTH];
	strncpy(DirNameHere, DirName, NV_MAX_PATHLENGTH-1);
#ifdef WIN32
	char FilesToFind[NV_MAX_PATHLENGTH];
	strncpy(FilesToFind, DirNameHere, NV_MAX_PATHLENGTH-1);
	if(strlen(FilesToFind))
	{
		// We add the last \ character
		if(FilesToFind[strlen(FilesToFind) - 1] != '\\')
			strcat(FilesToFind, "\\");
	}
	strcat(FilesToFind, "*");
	WIN32_FIND_DATA FindData;
	// Try to find the first file
	HANDLE hFileFind = FindFirstFile(FilesToFind, &FindData);
	if(hFileFind == INVALID_HANDLE_VALUE)
		return true;	// No file
	while(1)
	{
		if((FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0)
		{
			// File
			IsEmpty = false;
			break;
		}
		else
		{
			// Directory
			if(FindData.cFileName[0] != '.')
			{
				IsEmpty = false;
				break;
			}
		}

		if(!FindNextFile(hFileFind, &FindData))
			break;	// no more file

	}
	FindClose(hFileFind);	// we close the finder
#else

	char WorkDirHere[NV_MAX_PATHLENGTH];
	strncpy(WorkDirHere, DirNameHere, NV_MAX_PATHLENGTH-1);
	if(strlen(WorkDirHere))
	{
		// We remove the last / character
		if(WorkDirHere[strlen(WorkDirHere) - 1] == '/')
			WorkDirHere[strlen(WorkDirHere) - 1] = '\0';
	}
	else
		strcpy(WorkDirHere, ".");	// we are in the current directory

	// open directory file
	DIR *Dir = opendir(WorkDirHere);
	if(Dir == NULL)
		return true;		// cannot open directory file
	
	dirent *DirEntry;
	while((DirEntry = readdir(Dir)) != 0)
	{
		if(DirEntry->d_name[0] != '.')
		{
			// We have found a file or a directory
			IsEmpty = false;
			break;
		}
	}
	closedir(Dir);	// close the directory file
#endif

	return IsEmpty;
}


bool FileList(const char *DirName, const char *Root, vector<string> *List)
{
	if(DirName == NULL)
		return false;

	char DirNameHere[NV_MAX_PATHLENGTH];
	strncpy(DirNameHere, DirName, NV_MAX_PATHLENGTH-1);
	char FilesToFind[NV_MAX_PATHLENGTH];
#ifdef WIN32
	strncpy(FilesToFind, DirNameHere, NV_MAX_PATHLENGTH-1);
	if(strlen(FilesToFind))
	{
		// We add the last \ character
		if(FilesToFind[strlen(FilesToFind) - 1] != '\\')
			strcat(FilesToFind, "\\");
	}
	if(Root == NULL)
		strcat(FilesToFind, "*");
	else
		strcat(FilesToFind, Root);
	WIN32_FIND_DATA FindData;
	// Try to find the first file
	HANDLE hFileFind = FindFirstFile(FilesToFind, &FindData);
	if(hFileFind == INVALID_HANDLE_VALUE)
		return true;	// No file
	while(1)
	{
		// We remove all the found files
		if((FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0)
		{
			string FoundFileName = FindData.cFileName;
			List->push_back(FoundFileName);
		}

		if(!FindNextFile(hFileFind, &FindData))
			break;	// no more file

	}
	FindClose(hFileFind);	// we close the finder
#else

	string RootHere = "";
	if(Root != NULL)
		RootHere = Root;
	if(RootHere == "*")
		RootHere = "";
	char WorkDirHere[NV_MAX_PATHLENGTH];
	strncpy(WorkDirHere, DirNameHere, NV_MAX_PATHLENGTH-1);
	if(strlen(WorkDirHere))
	{
		// We remove the last / character
		if(WorkDirHere[strlen(WorkDirHere) - 1] == '/')
			WorkDirHere[strlen(WorkDirHere) - 1] = '\0';
	}
	else
		strcpy(WorkDirHere, ".");	// we are in the current directory

	// open directory file
	DIR *Dir = opendir(WorkDirHere);
	if(Dir == NULL)
		return true;		// cannot open directory file
	
	dirent *DirEntry;
	while((DirEntry = readdir(Dir)) != 0)
	{
		if(DirEntry->d_name[0] == '.')
			continue;
		strncpy(FilesToFind, WorkDirHere, NV_MAX_PATHLENGTH-1);
		if(strlen(FilesToFind))
		{
			// We add the last \ character
			if(FilesToFind[strlen(FilesToFind) - 1] != '/')
				strcat(FilesToFind, "/");
		}
		strcat(FilesToFind, DirEntry->d_name);
		DIR *DirHere = NULL;
		if((DirHere = opendir(FilesToFind)) != NULL)
		{
			// This is a directory
			closedir(DirHere);
			continue;
		}
		// We have found a real file
		string FoundFileName = DirEntry->d_name;
		if(RootHere.length())
			if(strstr(DirEntry->d_name, RootHere.c_str()) != DirEntry->d_name)
				continue;
		List->push_back(FoundFileName);	// This directory is a good candidate
	}
	closedir(Dir);	// close the directory file
#endif

	return true;
}




bool CreateDir(const char *DirName, bool LastDirOnly)
{
	if(DirName == NULL)
		return false;

	if(DirExist(DirName))
		return true;

	if(LastDirOnly)
	{

		// Only last directory
#ifdef WIN32
		if(mkdir(DirName) != 0)
#else
		if(mkdir(DirName, S_IRWXU | S_IRWXG | S_IRWXO) != 0)
#endif
			return false;
	}
	else
	{
		// All intermediate directories
		string::size_type index;

		string OriginDirName = DirName;
		CleanString(OriginDirName);

		if(!OriginDirName.length())
			return true;
		
		string BaseDir = "";
	#ifdef WIN32
		index = OriginDirName.find(':');
		//remove the drive letter for windows
		if(index != string::npos)
		{
			BaseDir = OriginDirName.substr(0,index+1);
			OriginDirName = OriginDirName.substr(index+1);
		}
		else
		{
			// Check if this is not a request to \\computer\dir
			// The first dir must then exist
			if(OriginDirName.find("\\\\") == 0)
			{
				index = OriginDirName.find_first_of("/\\", 2);
				if(index != string::npos)
				{
					index = OriginDirName.find_first_of("/\\", index+1);
					if(index != string::npos)
					{
						BaseDir = OriginDirName.substr(0,index);
						OriginDirName = OriginDirName.substr(index+1);
					}
					else
					{
						BaseDir = OriginDirName.substr(0,index);
						OriginDirName = "";
					}
				}
				else
				{
					BaseDir = OriginDirName;
					OriginDirName = "";
				}
			}
		}
		if(!OriginDirName.length())
			return true;
		while((index = OriginDirName.find('/')) != string::npos)
			OriginDirName.replace(index, 1, 1, '\\');
		if(OriginDirName[0] == '\\')
		{
			BaseDir += "\\";
			OriginDirName = OriginDirName.substr(1);
		}
	#else
		while((index = OriginDirName.find('\\')) != string::npos)
			OriginDirName.replace(index, 1, 1, '/');
		if(!OriginDirName.length())
			return true;
		if(OriginDirName[0] == '/')
		{
			BaseDir = "/";
			OriginDirName = OriginDirName.substr(1);
		}
	#endif
		if(!OriginDirName.length())
			return true;

		string DirNameHere = OriginDirName;
		string RealDirName = BaseDir;
		AddDirSuffix(RealDirName);

#ifdef WIN32
		while((index = DirNameHere.find('\\')) != string::npos)
#else
		while((index = DirNameHere.find('/')) != string::npos)
#endif
		{
			string SubDir = DirNameHere.substr(0, index);
			DirNameHere = DirNameHere.substr(index+1);
			string DirToCreate = RealDirName;
			DirToCreate += SubDir;
			if(!DirExist(DirToCreate.c_str()))
			{
#ifdef WIN32
				if(mkdir(DirToCreate.c_str()) != 0)
#else
				if(mkdir(DirToCreate.c_str(), S_IRWXU | S_IRWXG | S_IRWXO) != 0)
#endif
					return false;
			}
			RealDirName = DirToCreate;
			AddDirSuffix(RealDirName);
		}

		// Create last directory
		if(DirNameHere.length())
		{
			string DirToCreate = RealDirName;
			DirToCreate += DirNameHere;
			if(!DirExist(DirToCreate.c_str()))
			{
#ifdef WIN32
				if(mkdir(DirToCreate.c_str()) != 0)
#else
				if(mkdir(DirToCreate.c_str(), S_IRWXU | S_IRWXG | S_IRWXO) != 0)
#endif
					return false;
			}
		}
	}

	return true;
}



// Removes a directory and all its subdirectories and files
bool RemoveDir(const char *DirName, bool Self, const char *FileRoot, const char *SubDirRoot)
{
	if(DirName == NULL)
		return true;

	if(!DirExist(DirName))
		return true;

	bool WithFiles = true;
	bool WithSubDir = true;

	char FilesToRemove[NV_MAX_PATHLENGTH];
	char DirsToRemove[NV_MAX_PATHLENGTH];
	strcpy(FilesToRemove, "");
	strcpy(DirsToRemove, "");


	if(Self)
	{
		WithSubDir = true;
		WithFiles = true;
	}
	else
	{
		if(SubDirRoot != NULL)
		{
			if(strlen(SubDirRoot))
				strncpy(DirsToRemove, SubDirRoot, NV_MAX_PATHLENGTH-1);
			else
				WithSubDir = false;
		}
		if(FileRoot != NULL)
		{
			if(strlen(FileRoot))
				strncpy(FilesToRemove, FileRoot, NV_MAX_PATHLENGTH-1);
			else
				WithFiles = false;
		}
		if(strcmp(DirsToRemove, "*") == 0)
			strcpy(DirsToRemove, "");
		if(strcmp(FilesToRemove, "*") == 0)
			strcpy(FilesToRemove, "");
	}


	bool RetVal = true;
	char FileToRemove[NV_MAX_PATHLENGTH];
#ifdef WIN32
	// We use functions for finding files given a path and wildcards
	char FilesToFind[NV_MAX_PATHLENGTH];
	strncpy(FilesToFind, DirName, NV_MAX_PATHLENGTH-1);
	if(strlen(FilesToFind))
	{
		// We add the last \ character
		if(FilesToFind[strlen(FilesToFind) - 1] != '\\')
			strcat(FilesToFind, "\\");
	}
	strcat(FilesToFind, "*");
	WIN32_FIND_DATA FindData;
	// Try to find the first file
	HANDLE hFileFind = FindFirstFile(FilesToFind, &FindData);
	if(hFileFind == INVALID_HANDLE_VALUE)
		return false;	// No file
	while(1)
	{
		// We remove all the found files
		strncpy(FileToRemove, DirName, NV_MAX_PATHLENGTH-1);
		if(strlen(FileToRemove))
		{
			// We add the last \ character
			if(FileToRemove[strlen(FileToRemove) - 1] != '\\')
				strcat(FileToRemove, "\\");
		}
		strcat(FileToRemove, FindData.cFileName);
		if((FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0)
		{
			if(WithFiles)
			{
				bool ToRemove = false;
				if(strlen(FilesToRemove))
				{
					// Case insensitive for windows
					if(strstrnocase(FindData.cFileName, FilesToRemove) == FindData.cFileName)
						ToRemove = true;
				}
				else
					ToRemove = true;
				if(ToRemove)
					if(unlink(FileToRemove) != 0)
						RetVal = false;
			}
		}
		else if(FindData.cFileName[0] != '.')
		{
			if(WithSubDir)
			{
				bool ToRemove = false;
				if(strlen(DirsToRemove))
				{
					// Case insensitive for windows
					if(strstrnocase(FindData.cFileName, DirsToRemove) == FindData.cFileName)
						ToRemove = true;
				}
				else
					ToRemove = true;
				if(ToRemove)
					if(!RemoveDir(FileToRemove))
						RetVal = false;
			}
		}
		if(!FindNextFile(hFileFind, &FindData))
			break;	// no more file
	}
	FindClose(hFileFind);	// we close the finder
#else

	// for unix, we open the directory file and we search for entries
	// corresponding to our temporary file names
	char WorkDirHere[NV_MAX_PATHLENGTH];
	strcpy(WorkDirHere, DirName);
	if(strlen(WorkDirHere))
	{
		// We remove the last / character
		if(WorkDirHere[strlen(WorkDirHere) - 1] == '/')
			WorkDirHere[strlen(WorkDirHere) - 1] = '\0';
	}
	else
		strcpy(WorkDirHere, ".");	// we are in the current directory

	// open directory file
	DIR *Dir = opendir(WorkDirHere);
	if(Dir == NULL)
		return false;		// cannot open directory file
	
	dirent *DirEntry;
	while((DirEntry = readdir(Dir)) != 0)
	{
		// we remove files
		strncpy(FileToRemove, WorkDirHere, NV_MAX_PATHLENGTH-1);
		if(strlen(FileToRemove))
		{
			// We add the last \ character
			if(FileToRemove[strlen(FileToRemove) - 1] != '/')
				strcat(FileToRemove, "/");
		}
		strcat(FileToRemove, DirEntry->d_name);
		if(DirExist(FileToRemove))
		{
			// Its a directory
			if(WithSubDir && (DirEntry->d_name[0] != '.'))
			{
				bool ToRemove = false;
				if(strlen(DirsToRemove))
				{
					if(strstr(DirEntry->d_name, DirsToRemove) == DirEntry->d_name)
						ToRemove = true;
				}
				else
					ToRemove = true;
				if(ToRemove)
					if(!RemoveDir(FileToRemove))
						RetVal = false;
			}
		}
		else
		{
			//it's a file
			if(WithFiles)
			{
				bool ToRemove = false;
				if(strlen(FilesToRemove))
				{
					if(strstr(DirEntry->d_name, FilesToRemove) == DirEntry->d_name)
						ToRemove = true;
				}
				else
					ToRemove = true;
				if(ToRemove)
					if(unlink(FileToRemove) != 0)
						RetVal = false;
			}
		}

	}
	closedir(Dir);	// close the directory file
#endif


	if(Self)
		if(rmdir(DirName) != 0)
			return false;

	return RetVal;
}

bool CopyDir(const char *SrcDir, const char *DstDir, bool WithSubDir)
{
	// Check if source dir exists
	if(!DirExist(SrcDir))
		return false;

	// Create the destination directory
	if(!CreateDir(DstDir, false))
		return false;

	// List the source files
	vector<string> files;
	FileList(SrcDir, NULL, &files);

	// Process each file of the directory
	int i;
	for(i=0; i<(int)(files.size()); i++)
	{
		string SrcFile = SrcDir;
		AddDirSuffix(SrcFile);
		SrcFile += files[i];
		string DstFile = DstDir;
		AddDirSuffix(DstFile);
		DstFile += files[i];
		if(!CopyFileEx(SrcFile.c_str(), DstFile.c_str()))
			return false;
	}

	// Now eventually process the subdirectories
	if(WithSubDir)
	{
		vector<string> subdirs;
		DirList(SrcDir, NULL, &subdirs);

		// Process each subdirectory
		for(i=0; i<(int)(subdirs.size()); i++)
		{
			string SrcDirHere = SrcDir;
			AddDirSuffix(SrcDirHere);
			SrcDirHere += subdirs[i];

			string DstDirHere = DstDir;
			AddDirSuffix(DstDirHere);
			DstDirHere += subdirs[i];

			if(!CopyDir(SrcDirHere.c_str(), DstDirHere.c_str(), true))
				return false;
		}
	}
	return true;
}

bool GetDirLastModifiedCheckSum(string DirName, string& Result, bool WithSubDir)
{
	// Check if source dir exists
	if(!DirExist(DirName.c_str()))
		return false;

	// List the source files
	vector<string> files;
	FileList(DirName.c_str(), NULL, &files);

	// Process each file of the directory
	int i;
	for(i=0; i<(int)(files.size()); i++)
	{
		string SrcFile = DirName;
		AddDirSuffix(SrcFile);
		SrcFile += files[i];
		Result += files[i];
		Result += GetFileLastModifiedTime(SrcFile.c_str());
		Result = MD5_string(Result);
	}

	// Now eventually process the subdirectories
	if(WithSubDir)
	{
		vector<string> subdirs;
		DirList(DirName.c_str(), NULL, &subdirs);

		// Process each subdirectory
		for(i=0; i<(int)(subdirs.size()); i++)
		{
			string DirHere = DirName;
			AddDirSuffix(DirHere);
			DirHere += subdirs[i];

			if(!GetDirLastModifiedCheckSum(DirHere, Result, true))
				return false;
		}
	}

	return true;
}


int freadline(char *Line, int LineLength, FILE *stream)
{
	int LineLengthHere = LineLength-1;
	if(LineLength == 0)
		LineLengthHere = 0;

	if(fgets(Line, LineLengthHere, stream) <= 0)
		return 0;

	int RetVal = int(strlen(Line));

	if(strlen(Line) && (Line[strlen(Line)-1]) == '\n')
		Line[strlen(Line)-1] = '\0';
	if(strlen(Line) && (Line[strlen(Line)-1]) == '\r')
		Line[strlen(Line)-1] = '\0';

	return RetVal;
}


// Write data to a file on several lines by adding a \ character if data is greater
// than given max line length
size_t fwritelinemax(const char *data, size_t length, FILE* file, size_t maxlinelength)
{
	size_t totalnumwrites = 0;
	size_t numwrites = 0;
	if(length <= maxlinelength)
	{
		// line length is large enough
		totalnumwrites = fwrite(data, 1, length, file);
		return totalnumwrites;
	}

	// line length is not large enough
	int numblocks = (int)(length/maxlinelength);
	size_t lastblock = length%maxlinelength;

	int i;
	for(i=0;i<numblocks;i++)
	{
		numwrites = fwrite(data+totalnumwrites, 1, maxlinelength, file);
		totalnumwrites += numwrites;
		if(numwrites != maxlinelength)
			return totalnumwrites;
		if(fwrite("\\\n", 1, 2, file) != 2)
			return -1;
	}

	if(lastblock)
	{
		numwrites = fwrite(data+totalnumwrites, 1, lastblock, file);
		totalnumwrites += numwrites;
	}
	return totalnumwrites;
}



// Read a file until the StringToFind string is found
// Returns the actual offset value of the found string
// The file points to the start of the found string if found or to the end if not
// If the string is not found, the return value is -1
int findstring(FILE *stream, const char *StringToFind)
{
	int SizeString = (int)(strlen(StringToFind));
	if(SizeString == 0)
		return -1;
	int BufferSize = 1024 + SizeString;
	char *Buffer = (char*)malloc(BufferSize);
	if(Buffer == NULL)
		return -1;


	int RetVal = -1;

	int NumBytesRead;
	while((NumBytesRead = (int)fread(Buffer, 1, BufferSize-1, stream)) > 0)
	{
		Buffer[NumBytesRead] = '\0';
		char *buf = strstr(Buffer, StringToFind);
		if(buf != NULL)
		{
			int index = (int)(buf-Buffer);
			RetVal = ftell(stream) - NumBytesRead + index;
			fseek(stream, index-NumBytesRead, SEEK_CUR);
			break;
		}
		if(NumBytesRead < BufferSize-1)
			break;
		
		fseek(stream, 0-SizeString, SEEK_CUR);
	}

	free(Buffer);
	return RetVal;
}


// Read a binary data until the StringToFind string is found
// Returns the actual offset value of the found string from the start of the data
// If the string is not found, the return value is -1
int findstring(const char *Data, int DataSize, const char *StringToFind)
{
	int SizeString = (int)(strlen(StringToFind));
	if(SizeString == 0)
		return -1;
	if(SizeString > DataSize)
		return -1;

	int RetVal = -1;

	int i;
	for(i=0; i<DataSize; i++)
	{
		if(Data[i] != StringToFind[0])
			continue;
		// We have reached the first character
		int j;
		bool Found = true;
		for(j=1; j<SizeString; j++)
		{
			if(Data[i+j] != StringToFind[j])
			{
				Found = false;
				break;
			}
		}
		if(Found)
		{
			RetVal = i;
			break;
		}
	}
	return RetVal;
}

	
char *strstrnocase(const char *string, const char *strCharSet)
{
	if(string == NULL)
		return NULL;
	if(strCharSet == NULL)
		return NULL;

	char *stringhere = (char *)malloc(strlen(string)+1);
	char *strCharSethere = (char *)malloc(strlen(strCharSet)+1);
	strcpy(stringhere, string);
	strcpy(strCharSethere, strCharSet);
	strlwr(stringhere);
	strlwr(strCharSethere);
	char *buf = strstr(stringhere, strCharSethere);
	if(buf != NULL)
	{
		int index = (int)(buf-stringhere);
		buf = (char *)string + index;
	}
	free(stringhere);
	free(strCharSethere);

	return buf;
}


int strcmpnocase(const char *string1, const char *string2)
{
	if(string1 == NULL)
		return 0;
	if(string2 == NULL)
		return 0;

	char *string1here = (char *)malloc(strlen(string1)+1);
	char *string2here = (char *)malloc(strlen(string2)+1);
	strcpy(string1here, string1);
	strcpy(string2here, string2);
	strlwr(string1here);
	strlwr(string2here);
	int ret = strcmp(string1here, string2here);
	free(string1here);
	free(string2here);

	return ret;
}


int strncmpnocase(const char *string1, const char *string2, int len)
{
	if(string1 == NULL)
		return 0;
	if(string2 == NULL)
		return 0;

	char *string1here = (char *)malloc(strlen(string1)+1);
	char *string2here = (char *)malloc(strlen(string2)+1);
	strcpy(string1here, string1);
	strcpy(string2here, string2);
	strlwr(string1here);
	strlwr(string2here);
	int ret = strncmp(string1here, string2here, len);
	free(string1here);
	free(string2here);

	return ret;
}


void CleanString(char *String, bool NoSpace, bool ConcatSpace)
{
	if(String == NULL)
		return;
	if(!strlen(String))
		return;
	char *String2 = (char *)malloc(strlen(String) + 1);
	char *buf = String2;
	unsigned int i;
	for(i=0; i<strlen(String); i++)
	{
		if(String[i] == '\t')	// replace tab with space
			String[i] = ' ';
		if(String[i] == '\n')	// replace line feed with space
			String[i] = ' ';
		if(String[i] == '\r')	// eliminate carriage return
			continue;
		if(NoSpace)
			if(String[i] == ' ')	// eliminate space if required
				continue;
		if(ConcatSpace)
		{
			if(i && (String[i] == ' ') && (String[i-1] == ' '))	// concatenate 2 consecutive spaces
				continue;
		}
		if((buf == String2) && (String[i] == ' '))	// eliminate first space
			continue;
		*buf = String[i];
		buf++;
	}
	*buf = '\0';	// we end the string
	if(strlen(String2))			// we remove eventual last blank
		if(String2[strlen(String2)-1] == ' ')
			String2[strlen(String2)-1] = '\0';

	strcpy(String, String2);
	free(String2);

}
	
// make some cleaning on strings
void CleanString(string& StringToClean, bool NoSpace, bool ConcatSpace)
{
	if(!StringToClean.size())
		return;
	if(StringToClean.find_first_of("\t\r\n ") == string::npos)
		return;
	string::size_type index;
	if(StringToClean.find_first_of("\t\r\n") == string::npos)
	{
		if(NoSpace)
		{
			while((index = StringToClean.find(' ')) != string::npos)
				StringToClean.erase(index, 1);
		}
		else
		{
			while(StringToClean.size() && StringToClean[0] == ' ')
				StringToClean.erase(0, 1);
			// eliminate last space
			while(StringToClean.size() && StringToClean[StringToClean.size()-1] == ' ')
				StringToClean.erase(StringToClean.size()-1, 1);
			if(ConcatSpace)
			{
				while((index = StringToClean.find("  ")) != string::npos)
					StringToClean.replace(index, 2, " ");
			}
		}
		return;
	}

	
	// replace tab with space
	while((index = StringToClean.find('\t')) != string::npos)
		StringToClean.replace(index, 1, 1, ' ');
	// remove line feeds
	while((index = StringToClean.find('\n')) != string::npos)
		StringToClean.erase(index, 1);
	// remove carriage returns
	while((index = StringToClean.find('\r')) != string::npos)
		StringToClean.erase(index, 1);
	// concatenate 2 consecutive spaces
	if(ConcatSpace)
	{
		while((index = StringToClean.find("  ")) != string::npos)
			StringToClean.replace(index, 2, " ");
	}
	// eliminate first space
	if(StringToClean.size() && StringToClean[0] == ' ')
		StringToClean.erase(0, 1);
	// eliminate last space
	if(StringToClean.size() && StringToClean[StringToClean.size()-1] == ' ')
		StringToClean.erase(StringToClean.size()-1, 1);
	
	// remove spaces if required
	if(NoSpace)
	{
		while((index = StringToClean.find(' ')) != string::npos)
			StringToClean.erase(index, 1);
	}
	
}
	
// Check that a string contains only letter, number and underscore characters
bool CheckAsciiName(string& StringToCheck)
{
	int StringSize = (int)(StringToCheck.size());

	bool RetVal = true;
	int i;
	for(i=0; i<StringSize; i++)
	{
		if(StringToCheck[i] < 48)
		{
			RetVal = false;
			break;
		}
		if((StringToCheck[i] > 57) && (StringToCheck[i] < 65))
		{
			RetVal = false;
			break;
		}
		if((StringToCheck[i] > 90) && (StringToCheck[i] < 97) && (StringToCheck[i] != 95))
		{
			RetVal = false;
			break;
		}
		if(StringToCheck[i] > 122)
		{
			RetVal = false;
			break;
		}
	}

	return RetVal;
}


	

	
// make some cleaning on strings
void AddDirSuffix(string& DirName)
{
#ifdef WIN32
	if(DirName.size() && DirName[DirName.size()-1] != '\\')
		DirName += "\\";
#else
	if(DirName.size() && DirName[DirName.size()-1] != '/')
		DirName += "/";
#endif
}

	
// make some cleaning on strings
void RemoveDirSuffix(string& DirName)
{
	// eliminate last slash
	if(DirName.size() && DirName[DirName.size()-1] == '/')
		DirName.erase(DirName.size()-1, 1);
	// eliminate last backslash
	if(DirName.size() && DirName[DirName.size()-1] == '\\')
		DirName.erase(DirName.size()-1, 1);
}


	


bool CompareWorkDir(string Dir1, string Dir2)
{
	CleanString(Dir1);
	CleanString(Dir2);
	
	string::size_type index;
	// remove slashes
	while((index = Dir1.find('/')) != string::npos)
		Dir1.erase(index, 1);
	while((index = Dir2.find('/')) != string::npos)
		Dir2.erase(index, 1);
	// remove backslashes
	while((index = Dir1.find('\\')) != string::npos)
		Dir1.erase(index, 1);
	while((index = Dir2.find('\\')) != string::npos)
		Dir2.erase(index, 1);

#ifdef WIN32
	MakeLower(Dir1);
	MakeLower(Dir2);
#endif

	if(!Dir1.length())
		return false;

	if(!Dir2.length())
		return false;

	if(Dir1 == Dir2)
		return true;

	return false;
}

	
	

	
// Add /r and /n to a string
void AddEndLine(string& String)
{
#ifdef WIN32
	String += "\r";
#endif
	String += "\n";
}



void RemoveFirstAndEndBlanks(string& String)
{
	// eliminate first spaces
	while(String.size() && String[0] == ' ')
		String.erase(0, 1);
	// eliminate last spaces
	while(String.size() && String[String.size()-1] == ' ')
		String.erase(String.size()-1, 1);
}

	
void RemoveDoubleQuotes(string& String)
{
	RemoveChar(String, '\'');
}

	
void RemoveSimpleQuotes(string& String)
{
	RemoveChar(String, '\'');
}

	
void RemoveChar(string& String, char CharToRemove)
{
	string NewValue = "";
	while(1)
	{
		string::size_type index = String.find(CharToRemove);
		if(index == string::npos)
			break;
		NewValue += String.substr(0, index+1);
		if((index+1) >= String.size())
			break;
		String = String.substr(index+1);
		if(!String.length())
			break;
		if(String[0] == CharToRemove)
			String = String.substr(1);
	}
	NewValue += String;
	String = NewValue;
}


//**********************************************************************************
//**********************************************************************************
// Read information from an ini file
//**********************************************************************************
//**********************************************************************************
void GetFromProfile(const char *FileName, char *Output, unsigned int size, const char *Section, const char *Entry, const char *Default)
{
#ifdef WIN32
	// for windows, the function exists
	GetPrivateProfileString(Section, Entry, Default, Output, size, FileName);
#else
	strcpy(Output, Default);	// We write the default to the output
	char Line[256];
	char SectionHere[100];
	char EntryHere[100];
	// we prepare string for comparaison
//#ifdef AIX
//	sprintf(SectionHere, "[%s]", Section);
//#else
	snprintf(SectionHere, sizeof(SectionHere), "[%s]", Section);
//#endif
	strlwr(SectionHere);	// we translate section to lowercase for comparaison
//#ifdef AIX
//	sprintf(EntryHere, "%s=", Entry);
//#else
	snprintf(EntryHere, sizeof(EntryHere), "%s=", Entry);
//#endif
	strlwr(EntryHere);	// we translate entry to lowercase for comparaison
	FILE *stream = Nvfopen(FileName, "rt");
	if(stream == NULL)
		return;
	int SectionFound = 0;
	while(fgets(Line, sizeof(Line)-1, stream) > 0)
	{
		if(Line[strlen(Line)-1] == '\n')
			Line[strlen(Line)-1] = '\0';
		if(Line[strlen(Line)-1] == '\r')
			Line[strlen(Line)-1] = '\0';
		
		if(strlen(Line) == 0)			// read next line if line is blank
			continue;

		char *Line2 = (char *)malloc(strlen(Line) + 1);
		char *Line3 = (char *)malloc(strlen(Line) + 1);
		char *buf = Line2;
		unsigned int i;
		for(i=0; i<strlen(Line); i++)
		{
			if(Line[i] == '\t')	// replace tab with space
				Line[i] = ' ';
			if(i && (Line[i] == ' ') && (Line[i-1] == ' '))	// concatenate 2 consecutive spaces
				continue;
			if(i && (Line[i] == ' ') && (Line[i-1] == '='))	// remove space after '=' character
				continue;
			if(i && (Line[i] == ' ') && (Line[i-1] == '['))	// remove space after '[' character
				continue;
			if((Line[i] == ' ') && (Line[i+1] == '='))	// remove space before '=' character
				continue;
			if((Line[i] == ' ') && (Line[i+1] == '['))	// remove space before '[' character
				continue;
			if((Line[i] == ' ') && (Line[i+1] == ']'))	// remove space before ']' character
				continue;
			if((buf == Line2) && (Line[i] == ' '))	// eliminate first space
				continue;
			*buf++ = Line[i];
		}
		*buf = '\0';	// we end the string
		if(strlen(Line2))			// we remove eventual last blank
			if(Line2[strlen(Line2)-1] == ' ')
				Line2[strlen(Line2)-1] = '\0';
		strcpy(Line3, Line2);
		strlwr(Line3);			// Line3 is used for comparaison
		if(strlen(Line3) == 0)	// go to next line if this one is empty
		{
			free(Line2);
			free(Line3);
			continue;
		}
		
		if(SectionFound)
		{
			if(Line3[0] == '[')	// other section found
			{
				free(Line2);
				free(Line3);
				break;			// so break without success
			}
			if(strstr(Line3, EntryHere) == Line3)
			{
				// we found the entry
				// so we stop after getting the value
				strncpy(Output, Line2 + strlen(EntryHere), size);
				free(Line2);
				free(Line3);
				break;
			}
		}
		else
		{
			if(strstr(Line3, SectionHere) == Line3)
				SectionFound = 1;	// We have found the section
		}

		free(Line2);
		free(Line3);
	}
	
	fclose(stream);	       // close ini file
#endif
}





//**********************************************************************************
//**********************************************************************************
// removes the temporary files, if some exist, from the working directory
//**********************************************************************************
//**********************************************************************************
void RemoveFiles(const char *WorkDir, const char *Root)
{
	char FileToRemove[NV_MAX_PATHLENGTH];
#ifdef WIN32
	// We use functions for finding files given a path and wildcards
	char FilesToFind[NV_MAX_PATHLENGTH];
	strncpy(FilesToFind, WorkDir, NV_MAX_PATHLENGTH-1);
	if(strlen(FilesToFind))
	{
		// We add the last \ character
		if(FilesToFind[strlen(FilesToFind) - 1] != '\\')
			strcat(FilesToFind, "\\");
	}
	strcat(FilesToFind, Root);
	strcat(FilesToFind, "*");
	WIN32_FIND_DATA FindData;
	// Try to find the first file
	HANDLE hFileFind = FindFirstFile(FilesToFind, &FindData);
	if(hFileFind == INVALID_HANDLE_VALUE)
		return;	// No file
	while(1)
	{
		// We remove all the found files
		strcpy(FileToRemove, WorkDir);
		strcat(FileToRemove, FindData.cFileName);
		if((FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0)
			unlink(FileToRemove);
		if(!FindNextFile(hFileFind, &FindData))
			break;	// no more file
	}
	FindClose(hFileFind);	// we close the finder
#else

	// for unix, we open the directory file and we search for entries
	// corresponding to our temporary file names
	char WorkDirHere[NV_MAX_PATHLENGTH];
	strncpy(WorkDirHere, WorkDir, NV_MAX_PATHLENGTH-1);
	if(strlen(WorkDirHere))
	{
		// We remove the last / character
		if(WorkDirHere[strlen(WorkDirHere) - 1] == '/')
			WorkDirHere[strlen(WorkDirHere) - 1] = '\0';
	}
	else
		strcpy(WorkDirHere, ".");	// we are in the current directory

	// open directory file
	DIR *Dir = opendir(WorkDirHere);
	if(Dir == NULL)
		return;		// cannot open directory file
	
	dirent *DirEntry;
	bool AllFiles = false;
	if(strcmp(Root, "*") == 0)
		AllFiles = true;
	while((DirEntry = readdir(Dir)) != 0)
	{
		// we remove files
		if(AllFiles || (strstr(DirEntry->d_name, Root) == DirEntry->d_name))
		{
			strncpy(FileToRemove, WorkDir, NV_MAX_PATHLENGTH-1);
			strcat(FileToRemove, DirEntry->d_name);
			unlink(FileToRemove);
		}
	}
	closedir(Dir);	// close the directory file
#endif
}






//**********************************************************************************
//**********************************************************************************
// get the first file of a directory corresponding to the search string
//**********************************************************************************
//**********************************************************************************
string GetFirstFileDir(const char *WorkDir, const char *Root)
{
	string RetVal = "";
	char FileToSearch[NV_MAX_PATHLENGTH];
#ifdef WIN32
	// We use functions for finding files given a path and wildcards
	char FilesToFind[NV_MAX_PATHLENGTH];
	strncpy(FilesToFind, WorkDir, NV_MAX_PATHLENGTH-1);
	if(strlen(FilesToFind))
	{
		// We add the last \ character
		if(FilesToFind[strlen(FilesToFind) - 1] != '\\')
			strcat(FilesToFind, "\\");
	}
	strcat(FilesToFind, Root);
	strcat(FilesToFind, "*");
	WIN32_FIND_DATA FindData;
	// Try to find the first file
	HANDLE hFileFind = FindFirstFile(FilesToFind, &FindData);
	if(hFileFind == INVALID_HANDLE_VALUE)
		return RetVal;	// No file
	while(1)
	{
		strncpy(FileToSearch, WorkDir, NV_MAX_PATHLENGTH-1);
		strcat(FileToSearch, FindData.cFileName);
		if((FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0)
		{
			RetVal = FileToSearch;
			break;
		}
		if(!FindNextFile(hFileFind, &FindData))
			break;	// no more file
	}
	FindClose(hFileFind);	// we close the finder
#else

	// for unix, we open the directory file and we search for entries
	// corresponding to our temporary file names
	char WorkDirHere[NV_MAX_PATHLENGTH];
	strncpy(WorkDirHere, WorkDir, NV_MAX_PATHLENGTH-1);
	if(strlen(WorkDirHere))
	{
		// We remove the last / character
		if(WorkDirHere[strlen(WorkDirHere) - 1] == '/')
			WorkDirHere[strlen(WorkDirHere) - 1] = '\0';
	}
	else
		strcpy(WorkDirHere, ".");	// we are in the current directory

	// open directory file
	DIR *Dir = opendir(WorkDirHere);
	if(Dir == NULL)
		return RetVal;		// cannot open directory file
	
	dirent *DirEntry;
	bool AllFiles = false;
	if(strcmp(Root, "*") == 0)
		AllFiles = true;
	while((DirEntry = readdir(Dir)) != 0)
	{
		// we remove files
		if(AllFiles || (strstr(DirEntry->d_name, Root) == DirEntry->d_name))
		{
			strcpy(FileToSearch, WorkDir);
			strcat(FileToSearch, DirEntry->d_name);
			RetVal = FileToSearch;
			break;
		}
	}
	closedir(Dir);	// close the directory file
#endif

	return RetVal;
}








//**********************************************************************************
//**********************************************************************************
// returns the name of a file without reagrding its case
//**********************************************************************************
//**********************************************************************************
bool GetNoCaseRealFileName(string& FileName)
{
	bool RetVal = false;
	string FileNameHere = FileName;
	string DirName = ".";
	string::size_type index = FileNameHere.rfind('/');
	if(index == string::npos)
		index = FileNameHere.rfind('\\');
	if(index != string::npos)
	{
		DirName = FileNameHere.substr(0, index);
		FileNameHere = FileNameHere.substr(index+1);
	}
	MakeLower(FileNameHere);

#ifdef WIN32
	// We use functions for finding files given a path and wildcards
	char FilesToFind[NV_MAX_PATHLENGTH];
	strncpy(FilesToFind, DirName.c_str(), NV_MAX_PATHLENGTH-1);
	strcat(FilesToFind, "/");
	strcat(FilesToFind, "*");
	WIN32_FIND_DATA FindData;
	// Try to find the first file
	HANDLE hFileFind = FindFirstFile(FilesToFind, &FindData);
	if(hFileFind == INVALID_HANDLE_VALUE)
		return RetVal;	// No file
	while(1)
	{
		string FileInDir = FindData.cFileName;
		MakeLower(FileInDir);
		if((FileInDir == FileNameHere) && ((FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0))
		{
			FileName = DirName;
			FileName += "/";
			FileName += FindData.cFileName;
			RetVal = true;
			break;
		}
		if(!FindNextFile(hFileFind, &FindData))
			break;	// no more file
	}
	FindClose(hFileFind);	// we close the finder
#else

	// for unix, we open the directory file and we search for entries
	// corresponding to our temporary file names
	char WorkDirHere[NV_MAX_PATHLENGTH];
	strncpy(WorkDirHere, DirName.c_str(), NV_MAX_PATHLENGTH-1);

	// open directory file
	DIR *Dir = opendir(WorkDirHere);
	if(Dir == NULL)
		return RetVal;		// cannot open directory file
	
	dirent *DirEntry;
	while((DirEntry = readdir(Dir)) != 0)
	{
		if(DirEntry->d_name[0] == '.')
			continue;
		string FileInDir = DirEntry->d_name;
		MakeLower(FileInDir);
		if(FileInDir == FileNameHere)
		{
			strncpy(WorkDirHere, DirName.c_str(), NV_MAX_PATHLENGTH-1);
			if(strlen(WorkDirHere))
			{
				// We add the last \ character
				if(WorkDirHere[strlen(WorkDirHere) - 1] != '/')
					strcat(WorkDirHere, "/");
			}
			strcat(WorkDirHere, DirEntry->d_name);
			DIR *DirHere = NULL;
			if((DirHere = opendir(WorkDirHere)) != NULL)
			{
				// This is a directory
				closedir(DirHere);
				continue;
			}
			FileName = DirName;
			FileName += "/";
			FileName += DirEntry->d_name;
			RetVal = true;
			break;
		}
	}
	closedir(Dir);	// close the directory file
#endif

	return RetVal;
}



unsigned int GetAllocatedMemory()
{
#ifdef WIN32
	MEMORYSTATUS Status;
	GlobalMemoryStatus(&Status);
	return ((int)(Status.dwTotalVirtual - Status.dwAvailVirtual));
#endif
	return 0;	// for now for unix
}

unsigned int GetProcessAllocatedMemory()
{
	unsigned int RetVal = 0;

#ifdef NV_SERVER
#ifdef WIN32
#ifdef _DEBUG

	HANDLE hProcess;
    PROCESS_MEMORY_COUNTERS pmc;


    // Print information about the memory usage of the process.

    hProcess = OpenProcess(  PROCESS_QUERY_INFORMATION |
                                    PROCESS_VM_READ,
                                    FALSE, GetCurrentProcessId());
    if (NULL == hProcess)
        return 0;


    if(GetProcessMemoryInfo(hProcess, &pmc, sizeof(pmc)))
		RetVal = (unsigned int)(pmc.WorkingSetSize);

    CloseHandle( hProcess );

#endif
#endif
#endif
	return RetVal;
}




// Return codes: -1 = error, -2 = time out, 0 = OK
int ExecPrgm(const char *CommandLine, int *RetCode, int TimeOut)
{
	*RetCode = 0;

	int RetVal = 0;

	if(CommandLine == NULL)
		return -1;

	string CommandLineHere = CommandLine;
	CleanString(CommandLineHere, false, false);
	if(!CommandLineHere.length())
		return -1;


#ifdef WIN32
	char *buffer = NVNEW char[NV_MAX_PATHLENGTH];
	::GetCurrentDirectory(NV_MAX_PATHLENGTH, buffer);
	string CurrentDirectory = buffer;
	delete [] buffer;

	if(CurrentDirectory[CurrentDirectory.length() -1] != '\\')
		CurrentDirectory += "\\";

	string::size_type index;

	// we try to get the current directory from command line
	string Executable = CommandLineHere;
	if((Executable[0] == '\"') || (Executable[0] == '\''))
	{
		// The command probably contain spaces
		Executable.erase(0, 1);
		index = Executable.find('\"');
		if(index == string::npos)
			index = Executable.find('\'');
		if(index != string::npos)
			Executable = Executable.substr(0, index);
	
		// remove all double quotes
		while((index = Executable.find('\"')) != string::npos)
			Executable.erase(index, 1);
	
		// remove all quotes
		while((index = Executable.find('\'')) != string::npos)
			Executable.erase(index, 1);
	}
	else
	{
		// The command doesn't contain spaces
		// so the end of the executable is marked by an eventual space
		index = Executable.find(' ');
		if(index != string::npos)
			Executable = Executable.substr(0, index);
	}

	index = Executable.rfind('\\');
	if(index == string::npos)
		index = Executable.rfind('/');
	if(index != string::npos)
		CurrentDirectory = Executable.substr(0, index);


	buffer = NVNEW char[CommandLineHere.length() + 10];
	strcpy(buffer, CommandLineHere.c_str());

	
	PROCESS_INFORMATION ProcessInfo;

	STARTUPINFO StartInfo = {sizeof(STARTUPINFO), NULL, NULL, NULL,
					0, 0, 0, 0, 0, 0, 0,
					STARTF_USESHOWWINDOW, SW_HIDE,
					0, NULL, NULL, NULL, NULL};
	

	if(!CreateProcess(NULL, buffer, NULL, NULL,
						FALSE, NORMAL_PRIORITY_CLASS, NULL,
						CurrentDirectory.c_str(), &StartInfo,
						&ProcessInfo))
	{
		delete [] buffer;
		return -1;
	}
	
	delete [] buffer;

	DWORD ReturnCode;
	
	int CheckEvery = 1;
	time_t InitialTime = time(NULL);
	while(GetExitCodeProcess(ProcessInfo.hProcess, &ReturnCode))
	{
		SLEEP(10);
		if(ReturnCode != STILL_ACTIVE)
		{
			*RetCode = ReturnCode;
			break;
		}

		if(TimeOut == 0)
				break;		// No wait, returns immediatly
		if(TimeOut != -1)
		{
			time_t ActualTime = time(NULL);
			if((int)(ActualTime - InitialTime) > TimeOut)
			{
				// Time out, we try to kill the process
				TerminateProcess(ProcessInfo.hProcess, -1);
				RetVal = -2;	// Time out condition
				break;
			}
		}
		SLEEP(CheckEvery);

		// We first do the check very often but less and less while the time is growing
		CheckEvery += (CheckEvery/10)+1;
		if(CheckEvery > 10000)		// When the check time is 10 seconds, we check every 10 seconds
			CheckEvery = 10000;
	}
	CloseHandle(ProcessInfo.hProcess);
	CloseHandle(ProcessInfo.hThread);

#else

	int pid;
	int status;

	int BufferSize = 8192;
	char buffer[8192];
	strncpy(buffer, CommandLineHere.c_str(), BufferSize-1);
	buffer[BufferSize-1] = '\0';

	pid = fork();
	if(pid == -1)
		return -1;
	if(pid == 0)
	{
		// We are in the child
		char *argv[4];
		argv[0] = (char*)"sh";
		argv[1] = (char*)"-c";
		argv[2] = buffer;
		argv[3] = 0;
		execve("/bin/sh", argv, environ);
		exit(-1);
	}

	int CheckEvery = 1;
	time_t InitialTime = time(NULL);
	while(1)
	{
		
		SLEEP(10);
		pid_t child_pid = waitpid(pid, &status, WNOHANG | WUNTRACED);
		if(child_pid == -1)
		{
			RetVal = -1;	// Error
			break;
		}

		if(child_pid == pid)
		{
	
			// The process has terminated
			if(!WIFEXITED(status))
				RetVal = -1;	// Error running filter
			else if(WEXITSTATUS(status) == 127)
				RetVal = -1;	// Error running filter
			else
				*RetCode = WEXITSTATUS(status);	// Good return
			break;
		}

		if(TimeOut == 0)
				break;		// No wait, returns immediatly
		if(TimeOut != -1)
		{
			time_t ActualTime = time(NULL);
			if((int)(ActualTime - InitialTime) > TimeOut)
			{
				// Time out, we try to kill the process
				kill(pid, SIGTERM);
				waitpid(pid, &status, 0);	// We must wait to avoid zombies
				RetVal = -2;	// Time out condition
				break;
			}
		}
		SLEEP(CheckEvery);

		// We first do the check very often but less and less while the time is growing
		CheckEvery += (CheckEvery/10)+1;
		if(CheckEvery > 10000)		// When the check time is 10 seconds, we check every 10 seconds
			CheckEvery = 10000;
	}

#endif
	return RetVal;
}



// Loads a library
HMODULE MyLoadLibrary(const char *LibraryPath, bool Global)
{
	if(LibraryPath == NULL)
		return NULL;
	if(!strlen(LibraryPath))
		return NULL;
	
	int UnixFlags = 0;
#ifndef WIN32
	if(Global)
		UnixFlags = RTLD_NOW | RTLD_GLOBAL;
	else
		UnixFlags = RTLD_LAZY;
#endif
	HMODULE RetHandle = NULL;
#ifdef WIN32
	RetHandle = LoadLibrary(LibraryPath);
#else
	RetHandle = dlopen(LibraryPath, UnixFlags);
//#ifdef SOLARIS
//	RetHandle = dlopen(LibraryPath, RTLD_NOW | RTLD_GLOBAL | RTLD_GROUP);
//#else
//	RetHandle = dlopen(LibraryPath, RTLD_LAZY);
//#endif
#endif

	return RetHandle;
}


// Unload a library
void MyFreeLibrary(HMODULE hModule)
{
	if(hModule == NULL)
		return;

#ifdef WIN32
	FreeLibrary(hModule);
#else
	dlclose(hModule);
#endif
}


// Gets a pointer to a given function in a library
FARPROC MyGetLibFunction(HMODULE hModule, const char *FunctionName)
{
	if(hModule == NULL)
		return NULL;
	if(FunctionName == NULL)
		return NULL;
	if(!strlen(FunctionName))
		return NULL;

	FARPROC RetVal = NULL;
#ifdef WIN32
	RetVal = GetProcAddress(hModule, FunctionName);
#else
	RetVal = (FARPROC)dlsym(hModule, FunctionName);
#endif

	return RetVal;
}


// Encode 
// replaces some characters with special ones for xml
string EncodeStringForXML(string StringToEncode, bool strict)
{
	string Result = "";
	
	string::size_type index;
	if(strict)
	{
		// Search if there are characters less than 0x020
		// And remove them if not 0x09, 0x0A and 0x0D (these are authorized)
		int StringLength = (int)(StringToEncode.length());
		int i;
		bool HasBadChars = false;
		for(i=0;i<StringLength;i++)
		{
			if(((unsigned char)(StringToEncode[i])) < 0x20)
			{
				if((StringToEncode[i] != 0x09) && (StringToEncode[i] != 0x0A) && (StringToEncode[i] != 0x0D))
				{	
					HasBadChars = true;
					break;
				}
			}
		}
		if(HasBadChars)
		{
			char Buff[29] = {0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x0B,0x0C,0x0E,0x0F,0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x1A,0x1B,0x1C,0x1D,0x1E,0x1F,0x00};
			while((index = StringToEncode.find_first_of(Buff)) != string::npos)
				StringToEncode.erase(index, 1);
		}
	}
	
	if(!strict)
	{
		while((index = StringToEncode.find("&lt;")) != string::npos)
			StringToEncode.replace(index, 4, "<");
		while((index = StringToEncode.find("&gt;")) != string::npos)
			StringToEncode.replace(index, 4, ">");
		while((index = StringToEncode.find("&apos;")) != string::npos)
			StringToEncode.replace(index, 6, "\'");
		while((index = StringToEncode.find("&quot;")) != string::npos)
			StringToEncode.replace(index, 6, "\"");
		while((index = StringToEncode.find("&amp;")) != string::npos)
			StringToEncode.replace(index, 5, "&");
	}

	while((index = StringToEncode.find('&')) != string::npos)
		StringToEncode.replace(index, 1, "$$@@##%%amp;");
	while((index = StringToEncode.find('<')) != string::npos)
		StringToEncode.replace(index, 1, "&lt;");
	while((index = StringToEncode.find('>')) != string::npos)
		StringToEncode.replace(index, 1, "&gt;");
	while((index = StringToEncode.find('\'')) != string::npos)
		StringToEncode.replace(index, 1, "&apos;");
	while((index = StringToEncode.find('\"')) != string::npos)
		StringToEncode.replace(index, 1, "&quot;");
	while((index = StringToEncode.find("$$@@##%%")) != string::npos)
		StringToEncode.replace(index, 8, "&");

	return StringToEncode;
}

// Sets the file name for UNIX or WINDOWS
void SetDirName(std::string& DirName)
{
	string::size_type index;
	
#ifdef WIN32
	// replace \ with /
	while((index = DirName.find('/')) != string::npos)
		DirName.replace(index, 1, 1, '\\');
#else
	// replace \ with /
	while((index = DirName.find('\\')) != string::npos)
		DirName.replace(index, 1, 1, '/');
#endif
}


// Reformat a date
// returns false if the date format is not valid
// It always delivers a date on the format DD.MM.YYYY
bool ArrangeDate(std::string& DateToArrange)
{
	string DateToArrangeHere = DateToArrange;
	CleanString(DateToArrangeHere);
	MakeUpper(DateToArrangeHere);

	string::size_type index;
	// replace / with .
	while((index = DateToArrangeHere.find('/')) != string::npos)
		DateToArrangeHere.replace(index, 1, 1, '.');
	// replace space with .
	while((index = DateToArrangeHere.find(' ')) != string::npos)
		DateToArrangeHere.replace(index, 1, 1, '.');
	// replace \ with .
	while((index = DateToArrangeHere.find('\\')) != string::npos)
		DateToArrangeHere.replace(index, 1, 1, '.');
	// replace : with .
	while((index = DateToArrangeHere.find(':')) != string::npos)
		DateToArrangeHere.replace(index, 1, 1, '.');

	time_t tTime = time(NULL);
	struct tm tCurrentTime;
	localtime_r(&tTime, &tCurrentTime);
	int iCurrentDay = tCurrentTime.tm_mday;
	int iCurrentMonth = tCurrentTime.tm_mon + 1;
	int iCurrentYear = tCurrentTime.tm_year + 1900;

	char Buf[100];

	// If the date is not given, we take the actual date
	if(!DateToArrangeHere.length())
	{
		sprintf(Buf, "%02d.%02d.%04d", iCurrentDay, iCurrentMonth, iCurrentYear);
		DateToArrangeHere = Buf;
	}

	index = DateToArrangeHere.find('.');
	if(index == string::npos)
	{
		// We have no separator so we must check the content length
		if(DateToArrangeHere[0] == '-')
		{
			// We have x days before the current date
			string sNumDay = DateToArrangeHere.substr(1);
			int iNumDay = atoi(sNumDay.c_str());
			time_t tTime = time(NULL);
			tTime = tTime - (iNumDay * 86400);
			localtime_r(&tTime, &tCurrentTime);
			int iDay = tCurrentTime.tm_mday;
			int iMonth = tCurrentTime.tm_mon + 1;
			int iYear = tCurrentTime.tm_year + 1900;
			sprintf(Buf, "%02d.%02d.%04d", iDay, iMonth, iYear);
			DateToArrangeHere = Buf;
		}
		else if(DateToArrangeHere[0] == '+')
		{
			// We have x days after the current date
			string sNumDay = DateToArrangeHere.substr(1);
			int iNumDay = atoi(sNumDay.c_str());
			time_t tTime = time(NULL);
			tTime = tTime + (iNumDay * 86400);
			localtime_r(&tTime, &tCurrentTime);
			int iDay = tCurrentTime.tm_mday;
			int iMonth = tCurrentTime.tm_mon + 1;
			int iYear = tCurrentTime.tm_year + 1900;
			sprintf(Buf, "%02d.%02d.%04d", iDay, iMonth, iYear);
			DateToArrangeHere = Buf;
		}
		else
		{
			if(DateToArrangeHere.length() == 8)
			{
				// We suppose that we have a DDMMYYYY format
				string sDay = DateToArrangeHere.substr(0, 2);
				string sMonth = DateToArrangeHere.substr(2, 2);
				string sYear = DateToArrangeHere.substr(4, 4);
				int iDay = atoi(sDay.c_str());
				int iMonth = atoi(sMonth.c_str());
				int iYear = atoi(sYear.c_str());
				bool IsValid = true;
				if(iDay < 1)
					IsValid = false;
				if(iDay > 31)
					IsValid = false;
				if(iMonth < 1)
					IsValid = false;
				if(iMonth > 12)
					IsValid = false;
				if(iYear < 1970)
					IsValid = false;
				if(iYear > 2037)
					IsValid = false;
				if(!IsValid)
				{
						// Maybe we have YYYYMMDD
					sDay = DateToArrangeHere.substr(6, 2);
					sMonth = DateToArrangeHere.substr(4, 2);
					sYear = DateToArrangeHere.substr(0, 4);
					iDay = atoi(sDay.c_str());
					iMonth = atoi(sMonth.c_str());
					iYear = atoi(sYear.c_str());
					bool IsValid2 = true;
					if(iDay < 1)
						IsValid2 = false;
					if(iDay > 31)
						IsValid2 = false;
					if(iMonth < 1)
						IsValid2 = false;
					if(iMonth > 12)
						IsValid2 = false;
					if(iYear < 1970)
						IsValid2 = false;
					if(iYear > 2037)
						IsValid2 = false;
					if(IsValid2)
						IsValid = true;
				}
				if(!IsValid)
					return false;
				sprintf(Buf, "%02d.%02d.%04d", iDay, iMonth, iYear);
				DateToArrangeHere = Buf;
			}
			if(DateToArrangeHere.length() == 6)
			{
				// We suppose that we have a DDMMYY format
				string sDay = DateToArrangeHere.substr(0, 2);
				string sMonth = DateToArrangeHere.substr(2, 2);
				string sYear = DateToArrangeHere.substr(4, 2);
				int iDay = atoi(sDay.c_str());
				int iMonth = atoi(sMonth.c_str());
				int iYear = atoi(sYear.c_str());
				if(iYear < 100)
				{
					if(iYear < 50)
						iYear += 2000;
					else
						iYear += 1900;
				}
				bool IsValid = true;
				if(iDay < 1)
					IsValid = false;
				if(iDay > 31)
					IsValid = false;
				if(iMonth < 1)
					IsValid = false;
				if(iMonth > 12)
					IsValid = false;
				if(iYear < 1970)
					IsValid = false;
				if(iYear > 2037)
					IsValid = false;
				if(!IsValid)
				{
						// Maybe we have YYMMDD
					sDay = DateToArrangeHere.substr(4, 2);
					sMonth = DateToArrangeHere.substr(2, 2);
					sYear = DateToArrangeHere.substr(0, 2);
					iDay = atoi(sDay.c_str());
					iMonth = atoi(sMonth.c_str());
					iYear = atoi(sYear.c_str());
					if(iYear < 100)
					{
						if(iYear < 50)
							iYear += 2000;
						else
							iYear += 1900;
					}
					bool IsValid = true;
					if(iDay < 1)
						IsValid = false;
					if(iDay > 31)
						IsValid = false;
					if(iMonth < 1)
						IsValid = false;
					if(iMonth > 12)
						IsValid = false;
					if(iYear < 1970)
						IsValid = false;
					if(iYear > 2037)
						IsValid = false;
				}
				if(!IsValid)
					return false;
				sprintf(Buf, "%02d.%02d.%04d", iDay, iMonth, iYear);
				DateToArrangeHere = Buf;
			}
			if(DateToArrangeHere.length() == 4)
			{
				// We suppose that we have a DDMM format
				string sDay = DateToArrangeHere.substr(0, 2);
				string sMonth = DateToArrangeHere.substr(2, 2);
				int iDay = atoi(sDay.c_str());
				int iMonth = atoi(sMonth.c_str());
				int iYear = iCurrentYear;
				bool IsValid = true;
				if(iDay < 1)
					IsValid = false;
				if(iDay > 31)
					IsValid = false;
				if(iMonth < 1)
					IsValid = false;
				if(iMonth > 12)
					IsValid = false;
				if(!IsValid)
					return false;
				sprintf(Buf, "%02d.%02d.%04d", iDay, iMonth, iYear);
				DateToArrangeHere = Buf;
			}
			if(DateToArrangeHere.length() <= 2)
			{
				// We suppose that we have a DD format
				string sDay = DateToArrangeHere;
				int iDay = atoi(sDay.c_str());
				int iMonth = iCurrentMonth;
				int iYear = iCurrentYear;
				bool IsValid = true;
				if(iDay < 1)
					IsValid = false;
				if(iDay > 31)
					IsValid = false;
				if(!IsValid)
					return false;
				sprintf(Buf, "%02d.%02d.%04d", iDay, iMonth, iYear);
				DateToArrangeHere = Buf;
			}
		}
	}
	else
	{
		// We have at least a separator
		string sFirstVal = "";
		string sSecondVal = "";
		string sThirdVal = "";
		index = DateToArrangeHere.find('.');
		sFirstVal = DateToArrangeHere.substr(0, index);
		sSecondVal = DateToArrangeHere.substr(index+1);
		index = sSecondVal.find('.');
		if(index != string::npos)
		{
			sThirdVal = sSecondVal.substr(index+1);
			sSecondVal = sSecondVal.substr(0, index);
		}

		int iFirstVal = atoi(sFirstVal.c_str());
		int iSecondVal = atoi(sSecondVal.c_str());
		int iThirdVal = atoi(sThirdVal.c_str());

		if(iThirdVal > 0)
		{
			// We really have 3 parts
			// We must try to get the year part that is supposed to be the last one
			bool IsValid = true;
			if(iThirdVal < 100)
			{
				// Maybe the year is the first one
				if(iFirstVal > 100)
				{
					// Yes, this may be the case
					// so we revert them
					int i = iThirdVal;
					iThirdVal = iFirstVal;
					iFirstVal = i;
				}
			}
			if(iThirdVal < 100)
			{
				if(iThirdVal < 50)
					iThirdVal += 2000;
				else
					iThirdVal += 1900;
			}
			if(iThirdVal < 1970)
				IsValid = false;
			if(iThirdVal > 2037)
				IsValid = false;
			if(iFirstVal < 1)
				IsValid = false;
			if(iFirstVal > 31)
				IsValid = false;
			if(iSecondVal < 1)
				IsValid = false;
			if(iSecondVal > 12)
				IsValid = false;
			if(!IsValid)
				return false;

			sprintf(Buf, "%02d.%02d.%04d", iFirstVal, iSecondVal, iThirdVal);
			DateToArrangeHere = Buf;
		}
		else
		{
			// Only 2 values
			iThirdVal = iCurrentYear;
			sprintf(Buf, "%02d.%02d.%04d", iFirstVal, iSecondVal, iThirdVal);
			DateToArrangeHere = Buf;
		}
	}

	if(!DateToArrangeHere.length())
		return false;

	string Day = "";
	string Month = "";
	string Year = "";
	index = DateToArrangeHere.find('.');
	if(index != string::npos)
	{
		Day = DateToArrangeHere.substr(0, index);
		DateToArrangeHere = DateToArrangeHere.substr(index+1);
		Month = DateToArrangeHere;
	}
	index = DateToArrangeHere.find('.');
	if(index != string::npos)
	{
		Month = DateToArrangeHere.substr(0, index);
		Year = DateToArrangeHere.substr(index+1);
	}

	if(!Day.length())
		return false;
	if(!Month.length())
	{
		// Take current month
		sprintf(Buf, "%02d", iCurrentMonth);
	}
	if(!Year.length())
	{
		// Take current year
		sprintf(Buf, "%04d", iCurrentYear);
	}

	int iDay = atoi(Day.c_str());
	int iMonth = atoi(Month.c_str());
	int iYear = atoi(Year.c_str());

	if(iDay < 1)
		return false;
	if(iDay > 31)
		return false;

	if(iMonth < 1)
		return false;
	if(iMonth > 12)
		return false;

	if(iYear < 100)
	{
		if(iYear < 50)
			iYear += 2000;
		else
			iYear += 1900;
	}

	if(iYear < 1970)
		return false;
	if(iYear > 2037)
		return false;

	sprintf(Buf, "%02d.%02d.%04d", iDay, iMonth, iYear);

	DateToArrange = Buf;

	return true;
}


// Reformat an hour
// returns false if the date format is not valid
// It always delivers an hour on the format HHMMSS
bool ArrangeHour(std::string& HourToArrange)
{
	string HourToArrangeHere = HourToArrange;
	CleanString(HourToArrangeHere);
	MakeUpper(HourToArrangeHere);

	string::size_type index;
	// replace / with .
	while((index = HourToArrangeHere.find('/')) != string::npos)
		HourToArrangeHere.replace(index, 1, 1, '.');
	// replace space with .
	while((index = HourToArrangeHere.find(' ')) != string::npos)
		HourToArrangeHere.replace(index, 1, 1, '.');
	// replace \ with .
	while((index = HourToArrangeHere.find('\\')) != string::npos)
		HourToArrangeHere.replace(index, 1, 1, '.');
	// replace : with .
	while((index = HourToArrangeHere.find(':')) != string::npos)
		HourToArrangeHere.replace(index, 1, 1, '.');

	time_t tTime = time(NULL);
	struct tm tCurrentTime;
	localtime_r(&tTime, &tCurrentTime);
	int iCurrentHour = tCurrentTime.tm_hour;
	int iCurrentMin = tCurrentTime.tm_min;
	int iCurrentSec = tCurrentTime.tm_sec;

	char Buf[100];

	// If the Hour is not given, we take the actual Hour
	if(!HourToArrangeHere.length())
	{
		sprintf(Buf, "%02d.%02d.%04d", iCurrentHour, iCurrentMin, iCurrentSec);
		HourToArrangeHere = Buf;
	}

	index = HourToArrangeHere.find('.');
	if(index == string::npos)
	{
		// We have no separator so we must check the content length
		if(HourToArrangeHere[0] == '-')
		{
			// We have x Hours before the current Hour
			// Or x minutes
			string sNumMin = HourToArrangeHere.substr(1);
			bool MinutesRequired = false;
			if(sNumMin[sNumMin.length() - 1] == 'M')
			{
				MinutesRequired = true;
				sNumMin = sNumMin.substr(0, sNumMin.length() - 1);
			}
			int iNumMin = atoi(sNumMin.c_str());
			if(!MinutesRequired)
				iNumMin *= 60;	// hours required
			time_t tTime = time(NULL);
			tTime = tTime - (iNumMin * 60);
			localtime_r(&tTime, &tCurrentTime);
			int iHour = tCurrentTime.tm_hour;
			int iMin = tCurrentTime.tm_min;
			int iSec = tCurrentTime.tm_sec;
			sprintf(Buf, "%02d.%02d.%02d", iHour, iMin, iSec);
			HourToArrangeHere = Buf;
		}
		else if(HourToArrangeHere[0] == '+')
		{
			// We have x Hours after the current Hour
			// Or x minutes
			string sNumMin = HourToArrangeHere.substr(1);
			bool MinutesRequired = false;
			if(sNumMin[sNumMin.length() - 1] == 'M')
			{
				MinutesRequired = true;
				sNumMin = sNumMin.substr(0, sNumMin.length() - 1);
			}
			int iNumMin = atoi(sNumMin.c_str());
			if(!MinutesRequired)
				iNumMin *= 60;	// hours required
			time_t tTime = time(NULL);
			tTime = tTime + (iNumMin * 60);
			localtime_r(&tTime, &tCurrentTime);
			int iHour = tCurrentTime.tm_hour;
			int iMin = tCurrentTime.tm_min;
			int iSec = tCurrentTime.tm_sec;
			sprintf(Buf, "%02d.%02d.%02d", iHour, iMin, iSec);
			HourToArrangeHere = Buf;
		}
		else
		{
			if(HourToArrangeHere.length() == 6)
			{
				// We suppose that we have a HHMMSS format
				string sHour = HourToArrangeHere.substr(0, 2);
				string sMin = HourToArrangeHere.substr(2, 2);
				string sSec = HourToArrangeHere.substr(4, 2);
				int iHour = atoi(sHour.c_str());
				int iMin = atoi(sMin.c_str());
				int iSec = atoi(sSec.c_str());
				bool IsValid = true;
				if(iHour < 0)
					IsValid = false;
				if(iHour > 23)
					IsValid = false;
				if(iMin < 0)
					IsValid = false;
				if(iMin > 59)
					IsValid = false;
				if(iSec < 0)
					IsValid = false;
				if(iSec > 59)
					IsValid = false;
				if(!IsValid)
					return false;
				sprintf(Buf, "%02d.%02d.%02d", iHour, iMin, iSec);
				HourToArrangeHere = Buf;
			}
			if(HourToArrangeHere.length() == 4)
			{
				// We suppose that we have a HHMM format
				string sHour = HourToArrangeHere.substr(0, 2);
				string sMin = HourToArrangeHere.substr(2, 2);
				int iHour = atoi(sHour.c_str());
				int iMin = atoi(sMin.c_str());
				int iSec = 0;
				bool IsValid = true;
				if(iHour < 0)
					IsValid = false;
				if(iHour > 23)
					IsValid = false;
				if(iMin < 0)
					IsValid = false;
				if(iMin > 59)
					IsValid = false;
				if(!IsValid)
					return false;
				sprintf(Buf, "%02d.%02d.%02d", iHour, iMin, iSec);
				HourToArrangeHere = Buf;
			}
			if(HourToArrangeHere.length() <= 2)
			{
				// We suppose that we have a HH format
				string sHour = HourToArrangeHere;
				int iHour = atoi(sHour.c_str());
				int iMin = 0;
				int iSec = 0;
				bool IsValid = true;
				if(iHour < 0)
					IsValid = false;
				if(iHour > 23)
					IsValid = false;
				if(!IsValid)
					return false;
				sprintf(Buf, "%02d.%02d.%02d", iHour, iMin, iSec);
				HourToArrangeHere = Buf;
			}
		}
	}

	if(!HourToArrangeHere.length())
		return false;

	string Hour = HourToArrangeHere;
	string Min = "";
	string Sec = "";
	index = HourToArrangeHere.find('.');
	if(index != string::npos)
	{
		Hour = HourToArrangeHere.substr(0, index);
		HourToArrangeHere = HourToArrangeHere.substr(index+1);
		Min = HourToArrangeHere;
	}
	index = HourToArrangeHere.find('.');
	if(index != string::npos)
	{
		Min = HourToArrangeHere.substr(0, index);
		Sec = HourToArrangeHere.substr(index+1);
	}

	if(!Hour.length())
		return false;
	if(!Min.length())
		Min = "0";
	if(!Sec.length())
		Sec = "0";

	int iHour = atoi(Hour.c_str());
	int iMin = atoi(Min.c_str());
	int iSec = atoi(Sec.c_str());

	if(iHour < 0)
		return false;
	if(iHour > 23)
		return false;

	if(iMin < 0)
		return false;
	if(iMin > 59)
		return false;

	if(iSec < 0)
		return false;
	if(iSec > 59)
		return false;

	sprintf(Buf, "%02d%02d%02d", iHour, iMin, iSec);

	HourToArrange = Buf;

	return true;
}

void SetAsciiFile(const char *FileName)
{
	FILE *File = Nvfopen(FileName, "r");
	if(File == NULL)
		return;

	char *Buffer = (char *)malloc(65536);
	if(Buffer == NULL)
	{
		fclose(File);
		return;
	}

	if(fgets(Buffer, 65530, File) <= 0)
	{
		free(Buffer);
		fclose(File);
		return;
	}
	Buffer[65535] = '\0';

	bool bHasCarriageReturn = false;
	bool bHasLineFeed = false;

	if(Buffer[strlen(Buffer)-1] == '\n')
		bHasLineFeed = true;
	if(Buffer[strlen(Buffer)-1] == '\r')
		bHasCarriageReturn = true;
	
	fclose(File);

	if(!bHasLineFeed)
	{
		free(Buffer);
		return;
	}

	bool bProcessIt = false;
#ifdef WIN32
	if(!bHasLineFeed)
		bProcessIt = true;
#else
	if(bHasLineFeed)
		bProcessIt = true;
#endif

	if(!bProcessIt)
	{
		free(Buffer);
		return;
	}

	string TempFileName = FileName;
	TempFileName += ".nvtmpascii";


	FILE *FileSource;
	FILE *FileDest;
	

	FileSource = Nvfopen(FileName, "rb");
	if(FileSource == NULL)
	{
		free(Buffer);
		return;
	}

	FileDest = Nvfopen(TempFileName.c_str(), "wb");
	if(FileDest == NULL)
	{
		fclose(FileSource);
		RemoveFile(TempFileName.c_str());
		free(Buffer);
		return;
	}

	while(freadline(Buffer, 65530, FileSource))
	{
		Buffer[65530] = '\0';
#ifdef WIN32
		strcat(Buffer, "\r");
#endif
		strcat(Buffer, "\n");
		int BufLength = (int)(strlen(Buffer));
		if(fwrite(Buffer, 1, BufLength, FileDest)  != BufLength)
		{
			fclose(FileSource);
			fclose(FileDest);
			RemoveFile(TempFileName.c_str());
			free(Buffer);
			return;
		}
	}
	fclose(FileSource);
	fclose(FileDest);
	free(Buffer);

	CopyFileEx(TempFileName.c_str(), FileName);

	RemoveFile(TempFileName.c_str());
}




// replaces %xx in string by the character represented by
// the ASCII code xx
void DecodeUrlString(string& StringToDecode, bool bUnicode, bool bUrl, bool bAutoDetectEncoding)
{
	string::size_type index;

	if(bUrl)
	{
		// replace + with space
		while((index = StringToDecode.find('+')) != string::npos)
			StringToDecode.replace(index, 1, 1, ' ');

		// The length of the string must be at least 3
		if(StringToDecode.size() < 3)
			return;
		
		// We search for all occurences
		while((index = StringToDecode.find('%')) != string::npos)
		{
			// we do the job only if the % character is not one
			// of the 2 last characters
			if(index <= StringToDecode.size()-3)
			{
				// for the transformation we use both sscanf
				// and sprintf functions
				// with a temporary string containings
				// the 3 characters to manipulate
				unsigned int Value;
				char Zut[3];
				Zut[0] = StringToDecode[index+1];
				Zut[1] = StringToDecode[index+2];
				Zut[2] = '\0';
				sscanf(Zut, "%2x", &Value);
				sprintf(Zut,"%c", (unsigned char)Value); 
				if(Value == 0x25)
				{
					// % character itself
					string SpecialString = "$$##pourcent##$$";
					StringToDecode.replace(index, 3, SpecialString);
				}
				else
					StringToDecode.replace(index, 3, Zut);
			}
			else
				break;
		}
		while((index = StringToDecode.find("$$##pourcent##$$")) != string::npos)
			StringToDecode.replace(index, 16, "%");
	}

	if(!StringToDecode.length())
		return;

#ifdef NV_SERVER
	bool bDestinationUnicode = GetMain()->IsUnicode();
	if(bAutoDetectEncoding)
		bUnicode = IsUnicodeString(StringToDecode);
	EncodeString(StringToDecode, bUnicode, bDestinationUnicode);
#endif // NV_SERVER
}


// replaces some characters with %xx where xx is the ASCII code of the character
void EncodeUrlString(string& StringToEncode, bool bUnicode, bool bUrl)
{
	
	string::size_type index;
	if(bUrl)
	{
		while((index = StringToEncode.find('%')) != string::npos)
			StringToEncode.replace(index, 1, "$$@@##");
		while((index = StringToEncode.find("$$@@##")) != string::npos)
			StringToEncode.replace(index, 6, "%25");
		char c;
		char Zut[3];
		for(c=0; c <= 0x1F; c++)
		{
			sprintf(Zut,"%%%02x", c); 
			while((index = StringToEncode.find(c)) != string::npos)
				StringToEncode.replace(index, 1, Zut);
		}
		while((index = StringToEncode.find(0x7F)) != string::npos)
			StringToEncode.replace(index, 1, "%7F");
		while((index = StringToEncode.find('<')) != string::npos)
			StringToEncode.replace(index, 1, "%3C");
		while((index = StringToEncode.find('>')) != string::npos)
			StringToEncode.replace(index, 1, "%3E");
		while((index = StringToEncode.find('"')) != string::npos)
			StringToEncode.replace(index, 1, "%22");
		while((index = StringToEncode.find('#')) != string::npos)
			StringToEncode.replace(index, 1, "%23");
		while((index = StringToEncode.find('{')) != string::npos)
			StringToEncode.replace(index, 1, "%7B");
		while((index = StringToEncode.find('}')) != string::npos)
			StringToEncode.replace(index, 1, "%7D");
		while((index = StringToEncode.find('|')) != string::npos)
			StringToEncode.replace(index, 1, "%7C");
		while((index = StringToEncode.find('\\')) != string::npos)
			StringToEncode.replace(index, 1, "%5C");
		while((index = StringToEncode.find('^')) != string::npos)
			StringToEncode.replace(index, 1, "%5E");
		while((index = StringToEncode.find('~')) != string::npos)
			StringToEncode.replace(index, 1, "%7E");
		while((index = StringToEncode.find('[')) != string::npos)
			StringToEncode.replace(index, 1, "%5B");
		while((index = StringToEncode.find(']')) != string::npos)
			StringToEncode.replace(index, 1, "%5D");
		while((index = StringToEncode.find('`')) != string::npos)
			StringToEncode.replace(index, 1, "%60");
		while((index = StringToEncode.find(';')) != string::npos)
			StringToEncode.replace(index, 1, "%3B");
		while((index = StringToEncode.find('/')) != string::npos)
			StringToEncode.replace(index, 1, "%2F");
		while((index = StringToEncode.find('?')) != string::npos)
			StringToEncode.replace(index, 1, "%3F");
		while((index = StringToEncode.find(':')) != string::npos)
			StringToEncode.replace(index, 1, "%3A");
		while((index = StringToEncode.find('@')) != string::npos)
			StringToEncode.replace(index, 1, "%40");
		while((index = StringToEncode.find('=')) != string::npos)
			StringToEncode.replace(index, 1, "%3D");
		while((index = StringToEncode.find('&')) != string::npos)
			StringToEncode.replace(index, 1, "%26");
	}

	if(!StringToEncode.length())
		return;

#ifdef NV_SERVER
	bool bSourceUnicode = GetMain()->IsUnicode();
	EncodeString(StringToEncode, bSourceUnicode, bUnicode);
#endif // NV_SERVER
}

bool EncodeString(string& StringToEncode, bool bSrcUnicode, bool bDestUnicode)
{

	if(!StringToEncode.length())
		return true;

	if(bSrcUnicode == bDestUnicode)
		return true;		// nothing to do

#ifdef NV_SERVER

	bool RetVal = true;
	if(bSrcUnicode)
	{
		// convert from UTF-8 to latin1
		int BufferSize = ((int)(StringToEncode.length())) * 8;
		char *Buffer = (char *)(malloc(BufferSize+1));
		memset(Buffer, 0, BufferSize+1);
		int InLength =  (int)(StringToEncode.length());
		if(UTF8Toisolat1((unsigned char *)Buffer, &BufferSize, (const unsigned char *)StringToEncode.c_str(), &InLength) >= 0)
			StringToEncode = Buffer;
		else
		{
			RetVal = false;
			if(GetMain()->b_ConsoleMode)
			{
				NV_LOCKLOCK(&ProtectConsoleLog);
				printf("*** Error encoding from utf8 to iso latin. String: %s\n", StringToEncode.c_str()); 
				NV_UNLOCKLOCK(&ProtectConsoleLog);
			}
			GetMain()->Logs.Write("System", "Error encoding from utf8 to iso latin", StringToEncode.c_str(), NV_LOG_ERROR, 1, "", "system");
		}
		free(Buffer);
	}
	else
	{
		// convert from latin1 to UTF-8
		int BufferSize = ((int)(StringToEncode.length())) * 8;	// The buffer must be large enough
		char *Buffer = (char *)(malloc(BufferSize+1));
		memset(Buffer, 0, BufferSize+1);
		int InLength =  (int)(StringToEncode.length());
		if(isolat1ToUTF8((unsigned char *)Buffer, &BufferSize, (const unsigned char *)StringToEncode.c_str(), &InLength) >= 0)
			StringToEncode = Buffer;
		else
		{
			RetVal = false;
			if(GetMain()->b_ConsoleMode)
			{
				NV_LOCKLOCK(&ProtectConsoleLog);
				printf("*** Error encoding from iso latin 1 to utf8. String: %s\n", StringToEncode.c_str()); 
				NV_UNLOCKLOCK(&ProtectConsoleLog);
			}
			GetMain()->Logs.Write("System", "Error encoding from iso latin 1 to utf8", StringToEncode.c_str(), NV_LOG_ERROR, 1, "", "system");
		}

		free(Buffer);
	}

	return RetVal;
#endif // NV_SERVER
	return true;

}

// detect if latin1 or unicode
// Not very strict because the real latin1 characters 194 and 195 cannot be used ( and )
bool IsUnicodeString(string& StringToCheck)
{
/*	int i=0;
	unsigned const char* Data = (unsigned const char*)(StringToCheck.c_str());
	int Length = StringToCheck.length();

	for(i=0;i<Length;i++)
	{
		if(Data[i] == 195)
			return true;
		if(Data[i] == 194)
			return true;
	}

	return false;
*/
//#ifdef NV_SERVER
	if(utf8::is_valid(StringToCheck.begin(), StringToCheck.end()))
		return true;
//#endif

	return false;

}



// Next is for stgrptime implementation
static const char *weekdays[] = {
    "Sunday",
    "Monday",
    "Tuesday",
    "Wednesday",
    "Thursday",
    "Friday",
    "Saturday",
    NULL
};

static const char *month[] = {
    "January",
    "February",
    "Mars",
    "April",
    "May",
    "June",
    "July",
    "August",
    "September",
    "October",
    "November",
    "December",
    NULL,
};

static const char *ampm[] = {
    "am",
    "pm",
    NULL
};

/*
 * Try to match `*buf' to one of the strings in `strs'.  Return the
 * index of the matching string (or -1 if none).  Also advance buf.
 */

static int match_string (const char **buf, const char **strs, int ablen)
{
    int i = 0;

    for (i = 0; strs[i] != NULL; ++i) {
	int len = ablen > 0 ? ablen : (int)(strlen (strs[i]));

	if (strncmpnocase (*buf, strs[i], len) == 0) {
//	if (strncasecmp (*buf, strs[i], len) == 0) {
	    *buf += len;
	    return i;
	}
    }
    return -1;
}

/*
 * tm_year is relative this year 
 */

const int tm_year_base = 1900;

/*
 * Return TRUE iff `year' was a leap year.
 */

static int is_leap_year (int year)
{
    return (year % 4) == 0 && ((year % 100) != 0 || (year % 400) == 0);
}

/*
 * Return the weekday [0,6] (0 = Sunday) of the first day of `year'
 */

static int first_day (int year)
{
    int ret = 4;  // Jan 1, 1970 was on Thursday

    for (; year > 1970; --year)
	ret = (ret + 365 + is_leap_year (year) ? 1 : 0) % 7;
    return ret;
}

/*
 * Set `timeptr' given `wnum' (week number [0, 53])
 */

static void set_week_number_sun (struct tm *timeptr, int wnum)
{
    int fday = first_day (timeptr->tm_year + tm_year_base);

    timeptr->tm_yday = wnum * 7 + timeptr->tm_wday - fday;
    if (timeptr->tm_yday < 0) {
	timeptr->tm_wday = fday;
	timeptr->tm_yday = 0;
    }
}

/*
 * Set `timeptr' given `wnum' (week number [0, 53])
 */

static void set_week_number_mon (struct tm *timeptr, int wnum)
{
    int fday = (first_day (timeptr->tm_year + tm_year_base) + 6) % 7;

    timeptr->tm_yday = wnum * 7 + (timeptr->tm_wday + 6) % 7 - fday;
    if (timeptr->tm_yday < 0) {
	timeptr->tm_wday = (fday + 1) % 7;
	timeptr->tm_yday = 0;
    }
}

/*
 * Set `timeptr' given `wnum' (week number [0, 53])
 */

static void set_week_number_mon4 (struct tm *timeptr, int wnum)
{
    int fday = (first_day (timeptr->tm_year + tm_year_base) + 6) % 7;
    int offset = 0;

    if (fday < 4)
	offset += 7;

    timeptr->tm_yday = offset + (wnum - 1) * 7 + timeptr->tm_wday - fday;
    if (timeptr->tm_yday < 0) {
	timeptr->tm_wday = fday;
	timeptr->tm_yday = 0;
    }
}


char *mystrptime (const char *buf, const char *format, struct tm *timeptr)
{
    char c;

    timeptr->tm_yday = 1;  // So it's always valid
    timeptr->tm_isdst = 0;

    for (; (c = *format) != '\0'; ++format) {
	char *s;
	int ret;

	if (isspace (c)) {
	    while (isspace (*buf))
		++buf;
	} else if (c == '%' && format[1] != '\0') {
	    c = *++format;
	    if (c == 'E' || c == 'O')
		c = *++format;
	    switch (c) {
	    case 'A' :
		ret = match_string (&buf, weekdays, 0);
		if (ret < 0)
		    return NULL;
		timeptr->tm_wday = ret;
		break;
	    case 'a' :
		ret = match_string (&buf, weekdays, 3);
		if (ret < 0)
		    return NULL;
		timeptr->tm_wday = ret;
		break;
	    case 'B' :
		ret = match_string (&buf, month, 0);
		if (ret < 0)
		    return NULL;
		timeptr->tm_mon = ret;
		break;
	    case 'b' :
	    case 'h' :
		ret = match_string (&buf, month, 3);
		if (ret < 0)
		    return NULL;
		timeptr->tm_mon = ret;
		break;
	    case 'C' :
		ret = strtol (buf, &s, 10);
		if (s == buf)
		    return NULL;
		timeptr->tm_year = (ret * 100) - tm_year_base;
		buf = s;
		break;
	    case 'c' :
                // Date and Time in the current locale - unsupported
                return NULL;
	    case 'D' :		/* %m/%d/%y */
		s = mystrptime (buf, "%m/%d/%y", timeptr);
		if (s == NULL)
		    return NULL;
		buf = s;
		break;
	    case 'd' :
	    case 'e' :
		ret = strtol (buf, &s, 10);
		if (s == buf)
		    return NULL;
		timeptr->tm_mday = ret;
		buf = s;
		break;
	    case 'H' :
	    case 'k' :
		ret = strtol (buf, &s, 10);
		if (s == buf)
		    return NULL;
		timeptr->tm_hour = ret;
		buf = s;
		break;
	    case 'I' :
	    case 'l' :
		ret = strtol (buf, &s, 10);
		if (s == buf)
		    return NULL;
		if (ret == 12)
		    timeptr->tm_hour = 0;
		else
		    timeptr->tm_hour = ret;
		buf = s;
		break;
	    case 'j' :
		ret = strtol (buf, &s, 10);
		if (s == buf)
		    return NULL;
		timeptr->tm_yday = ret - 1;
		buf = s;
		break;
	    case 'm' :
		ret = strtol (buf, &s, 10);
		if (s == buf)
		    return NULL;
		timeptr->tm_mon = ret - 1;
		buf = s;
		break;
	    case 'M' :
		ret = strtol (buf, &s, 10);
		if (s == buf)
		    return NULL;
		timeptr->tm_min = ret;
		buf = s;
		break;
	    case 'n' :
		if (*buf == '\n')
		    ++buf;
		else
		    return NULL;
		break;
	    case 'p' :
		ret = match_string (&buf, ampm, 0);
		if (ret < 0)
		    return NULL;
		if (timeptr->tm_hour == 0) {
		    if (ret == 1)
			timeptr->tm_hour = 12;
		} else
		    timeptr->tm_hour += 12;
		break;
	    case 'r' :		/* %I:%M:%S %p */
		s = mystrptime (buf, "%I:%M:%S %p", timeptr);
		if (s == NULL)
		    return NULL;
		buf = s;
		break;
	    case 'R' :		/* %H:%M */
		s = mystrptime (buf, "%H:%M", timeptr);
		if (s == NULL)
		    return NULL;
		buf = s;
		break;
	    case 'S' :
		ret = strtol (buf, &s, 10);
		if (s == buf)
		    return NULL;
		timeptr->tm_sec = ret;
		buf = s;
		break;
	    case 't' :
		if (*buf == '\t')
		    ++buf;
		else
		    return NULL;
		break;
	    case 'T' :		/* %H:%M:%S */
	    case 'X' :
		s = mystrptime (buf, "%H:%M:%S", timeptr);
		if (s == NULL)
		    return NULL;
		buf = s;
		break;
	    case 'u' :
		ret = strtol (buf, &s, 10);
		if (s == buf)
		    return NULL;
		timeptr->tm_wday = ret - 1;
		buf = s;
		break;
	    case 'w' :
		ret = strtol (buf, &s, 10);
		if (s == buf)
		    return NULL;
		timeptr->tm_wday = ret;
		buf = s;
		break;
	    case 'U' :
		ret = strtol (buf, &s, 10);
		if (s == buf)
		    return NULL;
		set_week_number_sun (timeptr, ret);
		buf = s;
		break;
	    case 'V' :
		ret = strtol (buf, &s, 10);
		if (s == buf)
		    return NULL;
		set_week_number_mon4 (timeptr, ret);
		buf = s;
		break;
	    case 'W' :
		ret = strtol (buf, &s, 10);
		if (s == buf)
		    return NULL;
		set_week_number_mon (timeptr, ret);
		buf = s;
		break;
	    case 'x' :
		s = mystrptime (buf, "%Y:%m:%d", timeptr);
		if (s == NULL)
		    return NULL;
		buf = s;
		break;
	    case 'y' :
		ret = strtol (buf, &s, 10);
		if (s == buf)
		    return NULL;
		if (ret < 70)
		    timeptr->tm_year = 100 + ret;
		else
		    timeptr->tm_year = ret;
		buf = s;
		break;
	    case 'Y' :
		ret = strtol (buf, &s, 10);
		if (s == buf)
		    return NULL;
		timeptr->tm_year = ret - tm_year_base;
		buf = s;
		break;
	    case 'Z' :
                // Timezone spec not handled
                return NULL;
	    case '\0' :
		--format;
		/* FALLTHROUGH */
	    case '%' :
		if (*buf == '%')
		    ++buf;
		else
		    return NULL;
		break;
	    default :
		if (*buf == '%' || *++buf == c)
		    ++buf;
		else
		    return NULL;
		break;
	    }
	} else {
	    if (*buf == c)
		++buf;
	    else
		return NULL;
	}
    }
    return (char *)buf;
}


int axtoi(const char *hexStg)
{
  int n = 0;					// position in string
  int m = 0;					// position in digit[] to shift
  int count = (int)(strlen(hexStg));   // loop index
  if(count > 8)
	  count = 8;

  int intValue = 0;  // integer value of hex string
  int digit[9];      // hold values to convert
  for (n = 0; n < count; n++) {
     if (hexStg[n]=='\0')
        break;
     if (hexStg[n] > 0x29 && hexStg[n] < 0x40 ) //if 0 to 9
        digit[n] = hexStg[n] & 0x0f;            //convert to int
     else if (hexStg[n] >='a' && hexStg[n] <= 'f') //if a to f
        digit[n] = (hexStg[n] & 0x0f) + 9;      //convert to int
     else if (hexStg[n] >='A' && hexStg[n] <= 'F') //if A to F
        digit[n] = (hexStg[n] & 0x0f) + 9;      //convert to int
     else break;
  }
  count = n;
  m = n - 1;
  n = 0;
  while(n < count) {
     // digit[n] is value of hex digit at position n
     // (m << 2) is the number of positions to shift
     // OR the bits into return value
     intValue = intValue | (digit[n] << (m << 2));
     m--;   // adjust the position to set
     n++;   // next digit to process
  }
  return (intValue);
}

string CreateGuid()
{
	string Result = "";
#ifdef WIN32
#ifdef NV_SERVER
	UUID uid = {0};   
	if(UuidCreate(&uid) == RPC_S_OK)
	{
		PUCHAR pChar = 0;
		UuidToString( &uid, &pChar );
		Result = (const char *)pChar;
		RpcStringFree( &pChar );
	}
#endif
#endif
	MakeUpper(Result);
	return Result;
}

string MyGetFullPathName(const char* FileName)
{
	string Result = "";
#ifdef WIN32
	int BufferSize = 0;
	char *Buffer = NULL;
	char *FilePart;
	BufferSize = GetFullPathName(FileName, 0, NULL, &FilePart);
	if(BufferSize == 0)
		return Result;
	Buffer = (char *)malloc(BufferSize +1);
	if(GetFullPathName(FileName, BufferSize, Buffer, &FilePart))
		Result = Buffer;
	free(Buffer);
#else
	char Buffer[MAXPATHLEN];
	if(realpath(FileName, Buffer) != NULL)
		Result = Buffer;
#endif

	return Result;
}

string GetFileLastModifiedTime(const char* FileName)
{
	string Result = "";
	struct tm* tFileTime;					// create a time structure
	struct stat attrib;						// create a file attribute structure
    if(stat(FileName, &attrib) != 0)		// get the attributes of the file
		return Result;
	tFileTime = gmtime(&(attrib.st_mtime));	// Get the last modified time and put it into the time structure
	char sTime[200];
//	strftime(sTime, sizeof(sTime), "%a, %d %b %Y %H:%M:%S GMT", tFileTime);
	MyGetTime(sTime, tFileTime, true);
	Result = sTime;
	return Result;
}


string StripChars(string str, const string &chars)
{
	string result = "";
	int i;
	int length = (int)(str.length());
	for(i=0;i<length;i++)
	{
		if(chars.find(str[i]) == string::npos)
			result += str[i];
	}
    return result;
}

string KeepChars(string str, const string &chars)
{
	string result = "";
	int i;
	int length = (int)(str.length());
	for(i=0;i<length;i++)
	{
		if(chars.find(str[i]) != string::npos)
			result += str[i];
	}
    return result;
}

// Split following a separator, adds the string to result vector only if not empty
// Returns the number of strings found
int SplitString(string str, string sep, vector<string> *Splitted)
{
	Splitted->clear();
	string::size_type index;
	int lengthsep = (int)(sep.length());
	string Value = "";
	while((index = str.find(sep)) != string::npos)
	{
		Value = str.substr(0, index);
		if(Value.length())
			Splitted->push_back(Value);
		str = str.substr(index + lengthsep);
	}
	// Add last value
	if(str.length())
		Splitted->push_back(str);
	return (int)(Splitted->size());
}


#ifdef WIN32
#include <time.h>
#include <windows.h> 
#if defined(_MSC_VER) || defined(_MSC_EXTENSIONS)
	#define DELTA_EPOCH_IN_MICROSECS  11644473600000000Ui64
#else
	#define DELTA_EPOCH_IN_MICROSECS  11644473600000000ULL
#endif
struct timezone 
{
	int  tz_minuteswest; /* minutes W of Greenwich */
	int  tz_dsttime;     /* type of dst correction */
};
void l_getCurrentTime(long *sec, long *usec)
{
	ULARGE_INTEGER  utime, birthunix;
	FILETIME        systemtime;
	LONGLONG        birthunixhnsec = 116444736000000000;  /*in units of 100 ns */
	LONGLONG        usecs;

	GetSystemTimeAsFileTime(&systemtime);
	utime.LowPart  = systemtime.dwLowDateTime;
	utime.HighPart = systemtime.dwHighDateTime;

	birthunix.LowPart = (DWORD) birthunixhnsec;
	birthunix.HighPart = birthunixhnsec >> 32;

	usecs = (LONGLONG) ((utime.QuadPart - birthunix.QuadPart) / 10);

	if (sec) *sec = (long) (usecs /   1000000);
	if (usec) *usec = (long) (usecs % 1000000);
	return;
}
#endif
int Mygettimeofday(struct timeval *tv, struct timezone *tz)
{
#ifdef WIN32
//	FILETIME ft;
//	unsigned __int64 tmpres = 0;
	static int tzflag;

	if (NULL != tv)
	{
		l_getCurrentTime(&tv->tv_sec, &tv->tv_usec);
/*		GetSystemTimeAsFileTime(&ft);

		tmpres |= ft.dwHighDateTime;
		tmpres <<= 32;
		tmpres |= ft.dwLowDateTime;

		//converting file time to unix epoch
		tmpres -= DELTA_EPOCH_IN_MICROSECS; 
		tmpres /= 10;  //convert into microseconds
		tv->tv_sec = (long)(tmpres / 1000000UL);
		tv->tv_usec = (long)(tmpres % 1000000UL);*/
	}

	if (NULL != tz)
	{
		if (!tzflag)
		{
			_tzset();
			tzflag++;
		}
		tz->tz_minuteswest = _timezone / 60;
		tz->tz_dsttime = _daylight;
	}

	return 0;
#else
	return gettimeofday(tv, tz);
#endif
}


void SplitAddress(string &Address, int &Port)
{
	CleanString(Address);
	bool bIpV6 = false;
	const char *ptr = Address.c_str();
	// Count the number of ':'
	int NumCol = 0;
	while(*ptr != '\0')
	{
		if(*ptr == ':')
			NumCol++;
		ptr++;
	}
	if(NumCol==0)
		return;	// Nothing to do here (ipv4 or name without port)
	if(NumCol>=2)
		bIpV6 = true;
	if(bIpV6)
	{
		// For IpV6, the address must be enclosed into braquets if a port is specified
		if(Address[0] != '[')
			return; // Nothing to do here
		string::size_type index = Address.rfind(']');
		if(index == string::npos)
			return;	// This is an error we do nothing
		string sPort = Address.substr(index+1);
		Address = Address.substr(1, index-1);
		index = sPort.rfind(':');
		if(index == string::npos)
			return;	// Nothing more to do here
		sPort = sPort.substr(index+1);
		Port = atoi(sPort.c_str());
	}
	else
	{
		string::size_type index = Address.rfind(':');
		string sPort = Address.substr(index+1);
		Address = Address.substr(0, index);
		Port = atoi(sPort.c_str());
	}
}


void MyGetTime(char *Buffer, const struct tm *timeptr, bool bGMT)
{
	static const char wday_name[][4] = {
		"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
	};
	static const char mon_name[][4] = {
		"Jan", "Feb", "Mar", "Apr", "May", "Jun",
		"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
	};
	string sGMT = "";
	if(bGMT)
		sGMT  =" GMT";
	sprintf(Buffer, "%.3s, %02d %.3s %d %02d:%02d:%02d%s",
			wday_name[timeptr->tm_wday],
			timeptr->tm_mday,
			mon_name[timeptr->tm_mon],
			1900 + timeptr->tm_year,
			timeptr->tm_hour,
			timeptr->tm_min,
			timeptr->tm_sec,
			sGMT.c_str());
}

/* -*- mode: c; c-file-style: "k&r" -*-

strnatcmp.c -- Perform 'natural order' comparisons of strings in C.
Copyright (C) 2000, 2004 by Martin Pool <mbp sourcefrog net>

This software is provided 'as-is', without any express or implied
warranty.  In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software
in a product, an acknowledgment in the product documentation would be
appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.
*/


/* partial change history:
*
* 2004-10-10 mbp: Lift out character type dependencies into macros.
*
* Eric Sosman pointed out that ctype functions take a parameter whose
* value must be that of an unsigned int, even on platforms that have
* negative chars in their default char type.
*/

/*
* 2013-08-23: Skip leading zero's for any run of digits, except
*             when a decimal point was seen immediatley before.
*             Patch by LoRd_MuldeR <mulder2@gmx.de>
*/

#include <ctype.h>
#include <string.h>
#include <assert.h>
#include <stdio.h>

/* These are defined as macros to make it easier to adapt this code to
* different characters types or comparison functions. */
static inline int nat_isdigit(nat_char a)
{
	return iswdigit(a);
}

static inline int nat_isspace(nat_char a)
{
	return iswspace(a);
}

static inline nat_char nat_isdecpoint(nat_char a)
{
	return (a == L'.') || (a == L',');
}

static inline nat_char nat_toupper(nat_char a)
{
	return towupper(a);
}

static int compare_right(nat_char const *a, nat_char const *b)
{
	int bias = 0;

	/* The longest run of digits wins.  That aside, the greatest
	value wins, but we can't know that it will until we've scanned
	both numbers to know that they have the same magnitude, so we
	remember it in BIAS. */
	for (;; a++, b++)
	{
		if (!nat_isdigit(*a) && !nat_isdigit(*b))
			return bias;
		else if (!nat_isdigit(*a))
			return -1;
		else if (!nat_isdigit(*b))
			return +1;
		else if (*a < *b)
		{
			if (!bias)
				bias = -1;
		}
		else if (*a > *b)
		{
			if (!bias)
				bias = +1;
		}
		else if (!*a && !*b)
			return bias;
	}

	return 0;
}

static int compare_left(nat_char const *a, nat_char const *b)
{
	/* Compare two left-aligned numbers: the first to have a
	different value wins. */
	for (;; a++, b++)
	{
		if (!nat_isdigit(*a) && !nat_isdigit(*b))
			return 0;
		else if (!nat_isdigit(*a))
			return -1;
		else if (!nat_isdigit(*b))
			return +1;
		else if (*a < *b)
			return -1;
		else if (*a > *b)
			return +1;
	}

	return 0;
}

static int strnatcmp0(nat_char const *a, nat_char const *b, const bool fold_case)
{
	int ai, bi;
	nat_char ca, cb;
	int result;
	bool fractional;
	int sa, sb;
	
	assert(a && b);
	ai = bi = 0;
	fractional = false;

	while (1)
	{
		ca = a[ai]; cb = b[bi];

		/* skip over leading spaces or zeros */
		while (nat_isspace(ca))
			ca = a[++ai];

		while (nat_isspace(cb))
			cb = b[++bi];

		/* process run of digits */
		if (nat_isdigit(ca) && nat_isdigit(cb))
		{
			sa = sb = 0;

			if(!fractional)
			{
				while (ca == L'0')
				{
					ca = a[++ai]; sa++;
				}
				while (cb == L'0')
				{
					cb = b[++bi]; sb++;
				}
			}

			if (fractional)
			{
				if ((result = compare_left(a+ai, b+bi)) != 0)
					return result;
			}
			else
			{
				if ((result = compare_right(a+ai, b+bi)) != 0)
					return result;
			}

			/* on tie, the string with the longer leading zero's sequence wins */
			if(sa < sb)
				return -1;
			else if(sa > sb)
				return +1;
		}

		if (!ca && !cb)
		{
			/* The strings compare the same.  Perhaps the caller
			will want to call strcmp to break the tie. */
#ifdef WIN32
			return (fold_case) ? _wcsicmp(a, b) : wcscmp(a, b);
#else
			return (fold_case) ? wcscasecmp(a, b) : wcscmp(a, b);
#endif
		}

		if (fold_case)
		{
			ca = nat_toupper(ca);
			cb = nat_toupper(cb);
		}

		if (ca < cb)
			return -1;
		else if (ca > cb)
			return +1;

		/* skipp leading zero's, unless previously seen char was a decimal point */
		fractional = nat_isdecpoint(ca) && nat_isdecpoint(cb);

		++ai; ++bi;
	}
}

int strnatcmp(nat_char const *a, nat_char const *b)
{
	return strnatcmp0(a, b, false);
}

/* Compare, recognizing numeric string and ignoring case. */
int strnatcmpnocase(nat_char const *a, nat_char const *b)
{
	return strnatcmp0(a, b, true);
}



/* Compare, recognizing numeric string and ignoring case. Recognizes utf8. */
int strnatcmpnocase(const char *a, const char *b)
{
	bool bUnicode = false;
	string sa = a;
	string sb = b;
	if(IsUnicodeString(sa) || IsUnicodeString(sb))
		bUnicode = true;
	wstring wsa, wsb;
	if(bUnicode)
	{
		wsa = UTF8to16(a);
		wsb = UTF8to16(b);
	}
	else
	{
		char *provistr = iso88591_to_utf8(a);
		wsa = UTF8to16(provistr);
		free(provistr);
		provistr = iso88591_to_utf8(b);
		wsb = UTF8to16(provistr);
		free(provistr);
	}
	return strnatcmp0(wsa.c_str(), wsb.c_str(), true);
}

string UTF16to8(const wchar_t * in)
{
    std::string out;
    unsigned int codepoint = 0;
    for (in;  *in != 0;  ++in)
    {
        if (*in >= 0xd800 && *in <= 0xdbff)
            codepoint = ((*in - 0xd800) << 10) + 0x10000;
        else
        {
            if (*in >= 0xdc00 && *in <= 0xdfff)
                codepoint |= *in - 0xdc00;
            else
                codepoint = *in;

            if (codepoint <= 0x7f)
                out.append(1, static_cast<char>(codepoint));
            else if (codepoint <= 0x7ff)
            {
                out.append(1, static_cast<char>(0xc0 | ((codepoint >> 6) & 0x1f)));
                out.append(1, static_cast<char>(0x80 | (codepoint & 0x3f)));
            }
            else if (codepoint <= 0xffff)
            {
                out.append(1, static_cast<char>(0xe0 | ((codepoint >> 12) & 0x0f)));
                out.append(1, static_cast<char>(0x80 | ((codepoint >> 6) & 0x3f)));
                out.append(1, static_cast<char>(0x80 | (codepoint & 0x3f)));
            }
            else
            {
                out.append(1, static_cast<char>(0xf0 | ((codepoint >> 18) & 0x07)));
                out.append(1, static_cast<char>(0x80 | ((codepoint >> 12) & 0x3f)));
                out.append(1, static_cast<char>(0x80 | ((codepoint >> 6) & 0x3f)));
                out.append(1, static_cast<char>(0x80 | (codepoint & 0x3f)));
            }
            codepoint = 0;
        }
    }
    return out;
}

wstring UTF8to16(const char * in)
{
    std::wstring out;
    if (in == NULL)
        return out;

    unsigned int codepoint;
    while (*in != 0)
    {
        unsigned char ch = static_cast<unsigned char>(*in);
        if (ch <= 0x7f)
            codepoint = ch;
        else if (ch <= 0xbf)
            codepoint = (codepoint << 6) | (ch & 0x3f);
        else if (ch <= 0xdf)
            codepoint = ch & 0x1f;
        else if (ch <= 0xef)
            codepoint = ch & 0x0f;
        else
            codepoint = ch & 0x07;
        ++in;
        if (((*in & 0xc0) != 0x80) && (codepoint <= 0x10ffff))
        {
            if (codepoint > 0xffff)
            {
                out.append(1, static_cast<wchar_t>(0xd800 + (codepoint >> 10)));
                out.append(1, static_cast<wchar_t>(0xdc00 + (codepoint & 0x03ff)));
            }
            else if (codepoint < 0xd800 || codepoint >= 0xe000)
                out.append(1, static_cast<wchar_t>(codepoint));
        }
    }
    return out;
}


char* iso88591_to_utf8(const char *str)
{
    char *utf8 = (char*)(malloc(1 + (3 * strlen(str))));

    if (utf8) {
        char *c = utf8;
        for (; *str; ++str) {
            if (*str & 0x80) {
                *c++ = *str;
            } else {
                *c++ = (char) (0xc0 | (unsigned) *str >> 6);
                *c++ = (char) (0x80 | (*str & 0x3f));
            }
        }
        *c++ = '\0';
    }
    return utf8;
}